<?php /// BTW classes (PHP5-style)


// BTW data management

abstract class Cfg { // Configuration storage used in PDO operations

	// Attribute
	
	protected $config; // config
	
	// Constructor 
	
	function __construct($config) { 
		$this->config = $config;
	}
}

	
class Table extends Cfg { // Relational table in BTW database

	// Attributes
	
	public $name;
	
	// Methods

	function __construct($config, $name) { // Constructor 
		parent::__construct($config);
		$this->name = "btw_" . $name;
	}
	
	function create($attributes, $ifnotclause = 0) {
		$c = $this->config; require("db-connect.inc.php"); // $c used for db-connect.inc.php backward compatibility (ugly)
		$q = "create table ";
		if ($ifnotclause) $q .= "if not exists ";
		$q .= $this->name . " (";
		for ($i = 0; $i < count($attributes); $i++) {
			if ($i) $q .= ", ";
			$q .= $attributes[$i];
		}
		$q .= ")";
		$result = $cnx->query($q);
		if ($result === FALSE) return 0;
		else return 1;
	}
	
	function alter($attribute, $newtype, $action = "modify") {
		$c = $this->config; require("db-connect.inc.php");
		$q = "alter table $this->name ";
		if ($action == "modify") $q .= "change $attribute ";
		else $q .= "add ";
		$q .= "$attribute $newtype";
		$found = 0;
		if ($action == "add") {
			$result = $cnx->query("desc $this->name");
			while (($row = $result->fetch(PDO::FETCH_ASSOC)) && (!$found))
				if (strtolower($row["Field"]) == strtolower($attribute)) $found = 1;
			if ($found)	echo "<p><strong>Warning:</strong> row $attribute already exists; table $this->name alteration unnecessary.</p>\n\n";
		}
		if (!$found) {
			if ($cnx->query($q) === FALSE)
				echo "<p><strong>Error:</strong> Table $this->name could not be altered.</p>\n\n";
			else
				echo "<p>Table $this->name altered.</p>\n\n";
		}
	}
	
	function destroy() {
		$c = $this->config; require("db-connect.inc.php");
		if ($cnx->query("drop table $this->name") === FALSE) return 0;
		else return 1;
	}
	
	function select_name($fieldname = "", $oldid = "", $optional = 0) { // Select name from Asi and Paper tables
		$c = $this->config; require("db-connect.inc.php");
  		echo "<select name=\"$fieldname" . "name\">\n";
  		if ($optional) echo "<option></option>\n";
  		if ($oldid) {
			$result = $cnx->query("select name from " . $this->name . " where id = $oldid");
			$row = $result->fetch(PDO::FETCH_ASSOC);
			$oldname = $row["name"];
  		} else $oldname = "";
		foreach($cnx->query("select name from " . $this->name . " order by name") as $row) {
    		echo "<option";
    		if ($row["name"] == $oldname) echo " selected=\"selected\"";
		    echo ">" . $row["name"] . "</option>\n";
  		}
  		echo "</select>\n";
  		if ($optional) echo " <span class=\"small\">optional</span>\n";
	}	

	function get_new_id() { // ID calculation for tables with numerical ID
		$c = $this->config; require("db-connect.inc.php");
		$result = $cnx->query("select count(*) as c from $this->name");
		$row = $result->fetch(PDO::FETCH_ASSOC);
		if ($row["c"]) { // New ID = greatest ID + 1
			$result2 = $cnx->query("select max(id)+1 as m from $this->name");
			$row2 = $result2->fetch(PDO::FETCH_ASSOC);
			$id = $row2["m"];
		} else $id = 1; // Empty table, first ID is 1
		return $id;
	}
	
	function get_id($nametosearch) { // ID retrieval from name for tables with numerical ID
		$c = $this->config; require("db-connect.inc.php");
		$q = "select id from $this->name where name='" . addslashes($nametosearch) . "'";
		$result = $cnx->query($q);
		$row = $result->fetch(PDO::FETCH_ASSOC);
		return $row["id"];
	}	
	
	function count() {
		$c = $this->config; require("db-connect.inc.php");
		$result = $cnx->query("select count(*) as c from $this->name");
		$row = $result->fetch(PDO::FETCH_ASSOC);
		return $row["c"];
	}
}


class Config { // Configuration parameter set

	// Attributes (public)
	public $username;
	public $homepage;
	public $hostname;
	public $login;
	public $passwd;
	public $dbname;
	public $cssname;
	public $db_create;
	public $rsslength;
	public $emailalert;
	public $timezone;
	
	// Version info
	public $version;
	public $year;
	
	// Methods
	
	function __construct($username = "", $homepage = "", $hostname = "", $login = "", $passwd = "", $dbname = "", $cssname = "", $rsslength = "", $emailalert = "", $timezone = "") { // Constructor
		//
		$this->version = "4.03";
		$this->year = "2021";
		//
		if (!$username) require("config.inc.php");
		$this->username = $username;
		$this->homepage = $homepage;
		$this->hostname = $hostname;
		$this->login = $login;
		$this->passwd =	$passwd;
		$this->dbname = $dbname;
		$this->cssname = $cssname;
		$this->rsslength = $rsslength;
		$this->emailalert = $emailalert;
		$this->timezone = $timezone;
	}
	
	function save($db_create = 1) { // Create config file
		$fd = fopen("config.inc.php", "w");
		$result = fputs($fd, "<?php // BTW config file\n\$username = \"$this->username\";\n\$homepage = \"$this->homepage\";\n\$hostname = \"$this->hostname\";\n\$login = \"$this->login\";\n\$passwd = \"$this->passwd\";\n\$dbname = \"$this->dbname\";\n\$cssname = \"$this->cssname\";\n\$rsslength = $this->rsslength;\n\$emailalert = \"$this->emailalert\";\n\$timezone = \"$this->timezone\";\n?>");
		fclose($fd);
		if ($result) {
			$_SESSION["login"] = $this->login;
			$_SESSION["passwd"] = $this->passwd;
			echo "<p>Configuration file created.</p>\n";   
			if ($db_create) echo "<form action=\"" . $_SERVER["PHP_SELF"] . "\" method=\"post\"><fieldset><input type=\"hidden\" name=\"action\" value=\"create_db\"><input type=\"submit\" value=\"Proceed to database creation\"></fieldset></form>\n\n";
			else echo "<p>Back to <a href=\"" . $_SERVER["PHP_SELF"] . "\">installation menu</a>.</p>\n";
		} else echo "<p>Configuration file <strong>not</strong> created. <a href=\"" . $_SERVER["PHP_SELF"] . "\">Try again</a>!</p>\n";
	}
} 


class Database extends Cfg {

	// Create all tables in DB
	function create() { 
		$c = $this->config; require("db-connect.inc.php");
		$n = 28;
		$result = 0;
		// Paper
		unset($att);
		$att[0] = "bibkey varchar(15) primary key";
		$att[1] = "reference varchar(15)";
		$att[2] = "title varchar(255)";
		$att[3] = "yea year";
		$att[4] = "mon varchar(15)";
		$att[5] = "note varchar(255)";
		$att[6] = "scope varchar(15)";
		$att[7] = "abstract text";
		$att[8] = "keywords varchar(255)";
		$att[9] = "file varchar(50)";
		$att[10] = "url varchar(255)";
		$t = new Table($this->config, "paper");
		$result += $t->create($att);
		// Reference
		unset($att);
		$att[0] = "type varchar(15) primary key";
		$att[1] = "rank tinyint(2) unsigned";
		$t = new Table($this->config, "reference");
		$result += $t->create($att);
		$result += $cnx->exec("insert into $t->name values ('book', 5)");		
		$result += $cnx->exec("insert into $t->name values ('proceedings', 7)");		
		$result += $cnx->exec("insert into $t->name values ('article', 10)");
		$result += $cnx->exec("insert into $t->name values ('inbook', 15)");				
		$result += $cnx->exec("insert into $t->name values ('inproceedings', 20)");
		$result += $cnx->exec("insert into $t->name values ('phdthesis', 25)");
		$result += $cnx->exec("insert into $t->name values ('mastersthesis', 30)");
		$result += $cnx->exec("insert into $t->name values ('techreport', 35)");	
		$result += $cnx->exec("insert into $t->name values ('misc', 40)");	
		// Author
		unset($att);
		$att[0] = "id smallint(5) unsigned primary key";
		$att[1] = "name varchar(50)";
		$t = new Table($this->config, "author");
		$result += $t->create($att);		
		// Series
		unset($att);
		$att[0] = "id tinyint(3) unsigned primary key";
		$att[1] = "name varchar(100)";
		$att[2] = "url varchar(100)";
		$t = new Table($this->config, "series");
		$result += $t->create($att);
		// Institution
		unset($att);		
		$att[0] = "id tinyint(3) unsigned primary key";
		$att[1] = "name varchar(50)";
		$att[2] = "address varchar(50)";
		$att[3] = "url varchar(100)";
		$t = new Table($this->config, "institution");
		$result += $t->create($att);		
		// Misc
		unset($att);		
		$att[0] = "bibkey varchar(15) primary key";
		$att[1] = "howpublished varchar(255)";
		$t = new Table($this->config, "misc");
		$result += $t->create($att);		
		// Article
		unset($att);		
		$att[0] = "bibkey varchar(15) primary key";
		$att[1] = "journal tinyint(3) unsigned";
		$att[2] = "volume varchar(10)";
		$att[3] = "number varchar(10)";
		$att[4] = "pages varchar(15)";
		$t = new Table($this->config, "article");
		$result += $t->create($att);	
		// Book
		unset($att);		
		$att[0] = "bibkey varchar(15) primary key";
		$att[1] = "series tinyint(3) unsigned";
		$att[2] = "publisher tinyint(3) unsigned";		
		$att[3] = "volume varchar(10)";
		$att[4] = "edition varchar(15)";
		$t = new Table($this->config, "book");
		$result += $t->create($att);			
		// Inbook
		unset($att);		
		$att[0] = "bibkey varchar(15) primary key";
		$att[1] = "series tinyint(3) unsigned";
		$att[2] = "publisher tinyint(3) unsigned";		
		$att[3] = "volume varchar(10)";
		$att[4] = "pages varchar(15)";
		$att[5] = "edition varchar(15)";
		$t = new Table($this->config, "inbook");
		$result += $t->create($att);
		// Proceedings
		unset($att);	
		$att[0] = "bibkey varchar(15) primary key";
		$att[1] = "volume varchar(10)";
		$att[2] = "url varchar(100)";		
		$att[3] = "series tinyint(3) unsigned";
		$att[4] = "publisher tinyint(3) unsigned";	
		$att[5] = "organization tinyint(3) unsigned";			
		$t = new Table($this->config, "proceedings");
		$result += $t->create($att);		
		// Inproceedings
		unset($att);		
		$att[0] = "bibkey varchar(15) primary key";
		$att[1] = "series tinyint(3) unsigned";
		$att[2] = "publisher tinyint(3) unsigned";	
		$att[3] = "booktitle varchar(255)";
		$att[4] = "volume varchar(10)";
		$att[5] = "pages varchar(15)";
		$att[6] = "organization tinyint(3) unsigned";
		$att[7] = "url varchar(100)";
		$t = new Table($this->config, "inproceedings");
		$result += $t->create($att);	
		// Phdthesis
		unset($att);		
		$att[0] = "bibkey varchar(15) primary key";
		$att[1] = "school tinyint(3) unsigned";
		$t = new Table($this->config, "phdthesis");
		$result += $t->create($att);
		// Techreport
		unset($att);	
		$att[0] = "bibkey varchar(15) primary key";
		$att[1] = "institution tinyint(3) unsigned";
		$att[2] = "type varchar(50)";
		$att[3] = "number varchar(15)";
		$t = new Table($this->config, "techreport");
		$result += $t->create($att);			
		// Authors
		unset($att);	
		$att[0] = "bibkey varchar(15) not null";
		$att[1] = "id smallint(5) unsigned not null";
		$att[2] = "rank tinyint(2) unsigned";
		$att[3] = "primary key(bibkey, id)";
		$t = new Table($this->config, "authors");
		$result += $t->create($att);			
		// Editors (inproceedings editors)
		$t = new Table($this->config, "editors");
		$result += $t->create($att);
		// Proceedings editors
		$t = new Table($this->config, "proc_editors");
		$result += $t->create($att);
		// Book editors
		$t = new Table($this->config, "book_editors");
		$result += $t->create($att);					
		// Import (&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;)
		unset($att);	
		$att[0] = "id tinyint(3) unsigned primary key";
		$att[1] = "entry text";
		$t = new Table($this->config, "import");
		$result += $t->create($att);		
		// RSS feed
		unset($att);	
		$att[0] = "timestamp bigint(20) primary key";
		$att[1] = "title varchar(255)";
		$att[2] = "url varchar(255)";
		$att[3] = "description text";				
		$t = new Table($this->config, "rss");
		$result += $t->create($att);			
		// Check result
		if ($result == $n) echo "<p>Database created. Go to <a href=\"" . $_SERVER["PHP_SELF"] . "\">installation menu</a> or <a href=\"manage.php\">bibliography management</a></p>\n\n";
		else echo "<p>Database <strong>not</strong> fully created ($result operations completed out of $n). Try to <a href=\"" . $_SERVER["PHP_SELF"] . "\">destroy and recreate it</a>.</p>\n\n";		
	}

	// Destroy all tables in DB
	function destroy() { 
		$n = 19;
		$result = 0;
		$t = new Table($this->config, "paper"); $result += $t->destroy();
		$t = new Table($this->config, "reference"); $result += $t->destroy();
		$t = new Table($this->config, "author"); $result += $t->destroy();
		$t = new Table($this->config, "series"); $result += $t->destroy();
		$t = new Table($this->config, "institution"); $result += $t->destroy();
		$t = new Table($this->config, "misc"); $result += $t->destroy();
		$t = new Table($this->config, "article"); $result += $t->destroy();
		$t = new Table($this->config, "book"); $result += $t->destroy();
		$t = new Table($this->config, "inbook"); $result += $t->destroy();
		$t = new Table($this->config, "inproceedings"); $result += $t->destroy();
		$t = new Table($this->config, "phdthesis"); $result += $t->destroy();
		$t = new Table($this->config, "techreport"); $result += $t->destroy();
		$t = new Table($this->config, "authors"); $result += $t->destroy();
		$t = new Table($this->config, "editors"); $result += $t->destroy();
		$t = new Table($this->config, "import"); $result += $t->destroy();
		$t = new Table($this->config, "rss"); $result += $t->destroy();
		$t = new Table($this->config, "proceedings"); $result += $t->destroy();		
		$t = new Table($this->config, "book_editors"); $result += $t->destroy();	
		$t = new Table($this->config, "proc_editors"); $result += $t->destroy();	
		if ($result == $n) echo "<p>Database destroyed. ";
		else echo "<p>Database <strong>not</strong> fully destroyed ($result operations completed out of $n). ";
		echo "Back to <a href=\"" . $_SERVER["PHP_SELF"] . "\">installation menu</a>.</p>";
	}

	private function creation_result($res, $name) {
		if ($res) echo "<p>Table $name created (if absent).</p>\n\n";			
		else echo "<p><strong>Error:</strong> Table $name could not be created.</p>\n\n";		
	}
		
	// Upgrade DB to current version
	function upgrade() {
		// v1.2
		echo "<h2>Upgrade to version 1.2</h2>\n\n";
		unset($att);	
		$att[0] = "id tinyint(3) unsigned primary key";
		$att[1] = "entry text";
		$t = new Table($this->config, "import");
		$this->creation_result($t->create($att, 1), $t->name);
		// v2.1
		echo "<h2>Upgrade to version 2.1</h2>\n\n";		
		$t = new Table($this->config, "article");
		$t->alter("volume", "varchar(10)");
		$t = new Table($this->config, "book");
		$t->alter("volume", "varchar(10)");
		$t = new Table($this->config, "inbook");
		$t->alter("volume", "varchar(10)");
		$t = new Table($this->config, "inproceedings");
		$t->alter("volume", "varchar(10)");		
		$t = new Table($this->config, "paper");
		$t->alter("url", "varchar(255)", "add");	
		// v3.0
		echo "<h2>Upgrade to version 3.0</h2>\n\n";
		unset($att);	
		$att[0] = "timestamp bigint(20) primary key";
		$att[1] = "title varchar(255)";
		$att[2] = "url varchar(255)";
		$att[3] = "description text";				
		$t = new Table($this->config, "rss");
		$this->creation_result($t->create($att, 1), $t->name);
		// v3.1
		echo "<h2>Upgrade to version 3.1</h2>\n\n";
		// Authors & editors
		$t = new Table($this->config, "author");
		$t->alter("id", "smallint(5) unsigned");		
		$t = new Table($this->config, "authors");
		$t->alter("id", "smallint(5) unsigned not null");	
		$t = new Table($this->config, "editors");
		$t->alter("id", "smallint(5) unsigned not null");					
		// Proceedings item
		$c = $this->config; require("db-connect.inc.php");
		$result = $cnx->query("select count(*) as c from btw_reference where type='proceedings'");
		$row = $result->fetch(PDO::FETCH_ASSOC);
		if ($row["c"] == 0) {
			if ($cnx->query("insert into btw_reference values ('proceedings', 7)") == 0) echo "<p><strong>Error:</strong> Item proceedings could not be inserted.</p>\n\n";
			else echo "<p>Item proceedings inserted.</p>\n\n";
		} else echo "<p><strong>Warning:</strong> row proceedings already exists; table btw_reference alteration unnecessary.</p>\n\n";
		// Proceedings table
		unset($att);	
		$att[0] = "bibkey varchar(15) primary key";
		$att[1] = "volume varchar(10)";
		$att[2] = "url varchar(100)";		
		$att[3] = "series tinyint(3) unsigned";
		$att[4] = "publisher tinyint(3) unsigned";	
		$att[5] = "organization tinyint(3) unsigned";			
		$t = new Table($this->config, "proceedings");
		$this->creation_result($t->create($att, 1), $t->name);
		// Proceedings editor table
		unset($att);	
		$att[0] = "bibkey varchar(15) not null";
		$att[1] = "id smallint(5) unsigned not null";
		$att[2] = "rank tinyint(2) unsigned";
		$att[3] = "primary key(bibkey, id)";
		$t = new Table($this->config, "proc_editors");
		$this->creation_result($t->create($att, 1), $t->name);
		// Book editor table
		$t = new Table($this->config, "book_editors");
		$this->creation_result($t->create($att, 1), $t->name);
	}
}


class Rss extends Cfg{ // RSS feed

	// Attributes
	
	private $timestamp;
	private $title;
	private $url;
	private $description;
	
	// Methods
	
	function __construct($config, $timestamp, $title, $url, $description) { // Constructor for item
		parent::__construct($config);
		$this->timestamp = $timestamp;
		$this->title = $title;
		$this->url = $url;
		$this->description = $description;
	}
	
	function refresh($config) { // Refresh DB & feed -- private
		$c = $this->config; require("db-connect.inc.php");
		// Trim RSS table
		$result = $cnx->query("select count(*) as c from btw_rss");
		$row = $result->fetch(PDO::FETCH_ASSOC);
		for ($i = $row["c"]; $i > $c->rsslength; $i--) {
			$result2 = $cnx->query("select min(timestamp) as m from btw_rss");
			$row2 = $result2->fetch(PDO::FETCH_ASSOC);
			$cnx->exec("delete from btw_rss where timestamp=" . $row2["m"]);
		}
		// Regenerate RSS feed
		$rssuri = "https://" . $_SERVER["SERVER_NAME"] . dirname($_SERVER["REQUEST_URI"]) . "/files/btw-rss.xml";
		$fic = fopen("./files/btw-rss.xml", "w");
		fwrite($fic, "<?xml version=\"1.0\" encoding=\"utf-8\"?>\n\n");
		fwrite($fic, "<rss version=\"2.0\" xmlns:atom=\"http://www.w3.org/2005/Atom\">\n\n");
		fwrite($fic, "  <channel>\n\n");
		fwrite($fic, "    <title>Last publications of $config->username</title>\n");
		fwrite($fic, "    <link>$config->homepage</link>\n");
		fwrite($fic, "    <atom:link href=\"$rssuri\" rel=\"self\" type=\"application/rss+xml\" />\n");		
		fwrite($fic, "    <description>Generated by &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;Web $config->version</description>\n");
		fwrite($fic, "    <language>en</language>\n");	
		fwrite($fic, "    <lastBuildDate>" . date("r") . "</lastBuildDate>\n");
		foreach($cnx->query("select * from btw_rss order by timestamp desc") as $row) {
			fwrite($fic, "\n    <item>\n");
			fwrite($fic, "      <title>" . $row["title"] . "</title>\n");
			fwrite($fic, "      <link>" . $row["url"] . "</link>\n");
			fwrite($fic, "      <description>" . $row["description"] . "</description>\n");
			fwrite($fic, "      <guid>" . $row["url"] . "</guid>\n");
			fwrite($fic, "      <pubDate>" . date("r", $row["timestamp"]) . "</pubDate>\n");
			fwrite($fic, "    </item>\n");
		}
		fwrite($fic, "\n  </channel>\n\n</rss>\n");
		fclose($fic);	
	}
	
	function save($config) { // Save item into DB
		$c = $this->config; require("db-connect.inc.php");
		$q = "insert into btw_rss values($this->timestamp, '" . addslashes($this->title) . "', '$this->url', '" . addslashes($this->description) . "')";
		if ($cnx->exec($q) == 0) echo "<p><strong>Warning:</strong> Error when updating RSS table.</p>\n\n";
		else $this->refresh($config);
	}
}


class Paper extends Cfg { // Any paper

	// Attributes

	public $bibkey;
	public $title;
	public $scope;
	public $year;
	public $month;
	public $note;
	public $abstract;
	public $keywords;
	public $file;
	public $paperurl;

	public $bibtex;
	public $xml;

	// Associations

	public $reference;
	public $authors;
	
	// DBLP specifics
	private $dblpref;
	
	// Methods

	function __construct($config, $bibkey) { // Constructor
		parent::__construct($config);
		$this->bibkey = $bibkey;
		$this->reference = strtolower(get_class($this));
		$this->title = "";
		$this->scope = "";
		$this->year = date("Y");
		$this->month = "";
		$this->note = "";
		$this->abstract = "";
		$this->keywords = "";
		$this->file = "";
		$this->paperurl = "";
		$this->dblpref = $this->reference; // DBLP's DTD doesn't know some BibTeX entries, unfortunately...
	}
	
	function load() { // Load reference 
		$c = $this->config; require("db-connect.inc.php");
		$result = $cnx->query("select * from btw_paper, btw_reference where reference=type and bibkey='$this->bibkey'");
		$row = $result->fetch(PDO::FETCH_ASSOC);
    	$this->reference = $row["reference"];
	    $this->title = $row["title"];
    	$this->scope = $row["scope"];
    	$this->year = $row["yea"];
    	$this->month = $row["mon"];
    	$this->note = $row["note"];
    	$this->abstract = $row["abstract"];
    	$this->keywords = $row["keywords"];
    	$this->file = $row["file"];
    	$this->paperurl = $row["url"];
		$this->authors = array(); 
		$i = 0;
		foreach($cnx->query("select name from btw_author a, btw_authors b where a.id=b.id and bibkey='$this->bibkey' order by rank") as $row) $this->authors[$i++] = $row["name"];
		// BibTeX & XML computation
		$this->bibtex = "@" . strtoupper($this->reference) . "{" . $this->bibkey . ",<br />\n";
		$this->xml = "<". $this->dblpref . " key=\"" . $this->bibkey . "\"><br />\n";
		if (count($this->authors)) {
			$this->bibtex .= "&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;Author = {"; 
			$this->xml .= "&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<author>";
			for ($i = 0; $i < count($this->authors); $i++) {
		  		if ($i > 0) {
		    		$this->bibtex .= " and ";
		    		$this->xml .= ", ";
		  		}
		  		$this->bibtex .= $this->authors[$i];
		  		$this->xml .= $this->authors[$i];  
			}
			$this->bibtex .= "},<br />\n";
			$this->xml .= "</author><br />\n";
		}
		$this->bibtex .= "&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;Title = {" . $this->title . "},<br />\n";
		$this->xml .= "&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<title>" . $this->title ."</title><br />\n";
	}

	protected function load_specific_attributes() {
		$c = $this->config; require("db-connect.inc.php");
		$q = "select * from btw_$this->reference where bibkey='$this->bibkey'";
		$result = $cnx->query($q);
		return $result->fetch(PDO::FETCH_ASSOC);		
	}

	protected function bibtex_xml_footer() {
		if ($this->note) {
			$this->bibtex .= ",<br />\n&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;Note = {" . $this->note . "}";
			$this->xml .= "&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<note>$this->note</note><br />\n";
		}	
		if ($this->abstract) $this->bibtex .= ",<br />\n&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;Abstract = {" . $this->abstract . "}";
		if ($this->keywords) $this->bibtex .= ",<br />\n&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;Keywords = {" . $this->keywords . "}";		
		$this->bibtex .= "<br />\n}<br />\n\n";
		$this->xml .= "</$this->dblpref><br />\n\n";		
	}

	function display() {
		$d = "";
		if (count($this->authors)) {
			// Authors
			for ($i = 0; $i < count($this->authors); $i++) {
				if ($i > 0) $d .= ", ";
  				$namecomponents = explode(" ", $this->authors[$i]);
				$namecomponents2 = explode("-", $namecomponents[0]);
				for ($j=0; $j < count($namecomponents2); $j++) 
					$d .= strtoupper($namecomponents2[$j][0]) . ".";
				$d .= " " . $namecomponents[1];				
			}
			// Title
			if ($this->reference != "book") $d .= ", \"$this->title\"";
			else $d .= ", <em>$this->title</em>";
		} // Else it is an edited work that's processed in a subclass
		return $d;
	}
	
	protected function display_footer() {
		if ($this->note) $d = " (" . $this->note . ")";
		else $d = "";
		return $d . ".";
	}
	
	function announce($config) {
		$url = "https://" . $_SERVER["SERVER_NAME"] . dirname($_SERVER["REQUEST_URI"]) . "/#" . $this->bibkey;
		$dis = strip_tags($this->display("on"));
		// RSS feed
		$f = new Rss($this->config, time(), $this->title, $url, $dis);
		$f->save($config);
		// E-mail
		if ($config->emailalert) {
			$message = "\nNew publication: $dis\n\nDetails: $url\n\n";
			//echo "<p>Message sent by email:<br/>" . nl2br($message) . "</p>\n\n" ; // Debug
			mail($config->emailalert, "New publication for $config->username", $message);
		}
	}
	
	function form($action) { // Add-modify form
		$c = $this->config; require("db-connect.inc.php");
		echo "<form action=\"" . $_SERVER["PHP_SELF"] . "\" method=\"post\" enctype=\"multipart/form-data\"><fieldset>\n";
		echo "<input type=\"hidden\" name=\"action\" value=\"" . $action . "_ref\" />\n";
		//  Common fields part #1 
		if ($action == "add") echo "<h3>Key</h3>\n$this->bibkey";
		echo "<input type=\"hidden\" name=\"bibkey\" value=\"$this->bibkey\" />\n";
		if ($action == "add") echo "<h3>Reference type</h3>\n$this->reference";
		echo "<input type=\"hidden\" name=\"reference\" value=\"$this->reference\" />\n";
		echo "<h3>Scope</h3>\n<select name=\"scope\">\n<option>International</option>\n<option";
		if ($this->scope == "National") echo " selected=\"selected\"";
		echo ">National</option></select>\n";
		if ($this->reference != "proceedings") {
			echo "<h3>Author(s)</h3>\n<ul>";
			foreach($cnx->query("select name from btw_author as a, btw_authors as b where a.id=b.id and bibkey='$this->bibkey'") as $row) echo "<li>" . $row["name"] . "</li>\n";
			echo "</ul>\n";
		}
		echo "<h3>Title</h3>\n<input type=\"text\" name=\"title\" size=\"50\" maxlength=\"255\" value=\"$this->title\" />\n";
		echo "<h3>Year</h3>\n<input type=\"text\" name=\"year\" size=\"4\" maxlength=\"4\" value=\"$this->year\" />\n";
		echo "<h3>Month</h3>\n<input type=\"text\" name=\"month\" size=\"15\" maxlength=\"15\" value=\"$this->month\" /> <span class=\"small\">optional</span>\n";	
	}
	
	protected function form_footer() { // Add-modify form part #2
		// Common fields part #2 
		echo "<h3>Note</h3>\n<input type=\"text\" name=\"note\" size=\"50\" maxlength=\"255\" value=\"$this->note\" />  <span class=\"small\">optional</span>\n";
		echo "<h3>Abstract</h3>\n<textarea name=\"abstract\" rows=\"10\" cols=\"40\">$this->abstract</textarea>  <span class=\"small\">optional</span>\n";
		echo "<h3>Keywords</h3>\n<input type=\"text\" name=\"keywords\" size=\"50\" maxlength=\"255\" value=\"$this->keywords\" />   <span class=\"small\">optional</span>\n";
		echo "<h3>Full paper file</h3>\n";
		if ($this->file) echo "<p><tt>$this->file</tt></p>\n";
		echo "<p>Leave empty if file is not modified.</p>\n";
		echo "<input type=\"hidden\" name=\"old_file\" value=\"$this->file\" />\n";
		echo "<input type=\"file\" name=\"file\" size=\"50\" maxlength=\"255\" /> <span class=\"small\">optional</span>\n";
		echo "<h3>Paper URL</h3>\n";
		echo "<p class=\"small\">(If paper is available externally only).</p>\n";
		echo "<input type=\"text\" name=\"paperurl\" size=\"50\" maxlength=\"255\" value=\"$this->paperurl\" />  <span class=\"small\">optional</span>\n";
		//  Buttons 
		echo "<p><input type=\"reset\" name=\"Reset\" value=\"Reset\" />\n";
		echo "<input type=\"submit\" name=\"Submit\" value=\"Submit\" /></p>\n";
		echo "</fieldset></form>\n\n";
	}
	
	function upload($tmpfile, $file_name, $old_file) { // File upload
		//echo "<p>UPLOAD: \$tmpfile=$tmpfile - \$file_name=$file_name - \$old_file=$old_file</p>\n"; // Debug
		if ($file_name) { // Upload file
			$bd_name = $file_name;
			$dest = "files/$bd_name";
			$result = move_uploaded_file($tmpfile, $dest);
			if (($tmpfile=='none') || ($result!=1)) echo "<p><strong>Warning:</strong> File transfer problem for <tt>$file_name</tt>.</p>\n\n";
			else {
				if (($old_file) && ($old_file != $bd_name)) { // Delete old file
					$todel = "files/$old_file";
					unlink($todel);	
				}
			}
		} else $bd_name = $old_file;
		return $bd_name;
	}
	
	function save($title, $year, $month, $note, $scope, $abstract, $keywords, $tmpfile, $filename, $oldfile, $paperurl, $journalname, $volume, $number, $pages, $seriesname, $publishername, $edition, $organizationname, $booktitle, $url, $howpublished, $schoolname, $institutionname, $type) { // Save reference into DB
		$c = $this->config; require("db-connect.inc.php");
		// Upload file
		$file = $this->upload($tmpfile, $filename, $oldfile);
		$result = $cnx->exec("insert into btw_paper values ('$this->bibkey', '$this->reference', '" . addslashes($title) . "', $year, '$month', '" . addslashes($note) . "', '$scope', '" . addslashes($abstract) . "', '" . addslashes($keywords) . "', '$file', '$paperurl')");
		if ($result === FALSE) return 0;
		else return $result;
	}
	
	function form_update($current_display, $subaction, $name) { // Update form
		$c = $this->config; require("db-connect.inc.php");
	    // Key and ref type
		echo "<form action=\"" . $_SERVER["PHP_SELF"] . "\" method=\"post\" enctype=\"multipart/form-data\"><fieldset>\n";
		echo "<input type=\"hidden\" name=\"oldkey\" value=\"$this->bibkey\" />\n";
		echo "<input type=\"hidden\" name=\"action\" value=\"modify_key_type\" />\n";
		echo "<h3>Reference key</h3>\n";
		echo "<input type=\"text\" name=\"bibkey\" size=\"15\" maxlength=\"15\" value=\"$this->bibkey\" />\n";
		echo "<h3>Reference type</h3>\n<select name=\"reference\">\n";
		foreach($cnx->query("select type from btw_reference order by type") as $row) {
			echo "<option";
			if ($row["type"] == $this->reference) echo " selected=\"selected\"";
			echo ">" . $row["type"] . "</option>\n";
		}
		echo "</select>\n";
		echo "<p><input type=\"reset\" name=\"Reset\" value=\"Reset\" />\n";
		echo "<input type=\"submit\" name=\"Submit\" value=\"Submit\" /></p>\n";
		echo "</fieldset></form>\n\n<hr />\n\n";
		// Authors and editors
		if ($this->reference != "proceedings") // Authors
			$current_display->select_authed("form_modify_ref", "authors", "", $this->bibkey, $this->reference, $subaction, $name);
		else // Proceedings editors
			$current_display->select_authed("form_modify_ref", "proc_editors", "", $this->bibkey, $this->reference, $subaction, $name);
		echo "<hr />\n\n";
		if (($this->reference == "inproceedings") || ($this->reference == "book")) { // Inproceedings/book editors
			if ($this->reference == "inproceedings") $table = "editors";
			else $table = "book_editors";
			$current_display->select_authed("form_modify_ref", $table, "", $this->bibkey, $this->reference, $subaction, $name);
			echo "<hr />\n\n";
		}	
		// All the rest!
		$this->form("modify");	
	}
	
	function update_key($newkey) { 
		$c = $this->config; require("db-connect.inc.php");
		if ($cnx->exec("update btw_paper set bibkey='$newkey', reference='$this->reference' where bibkey='$this->bibkey'") === FALSE) $result = 0;
		else $result = 1;
		if ($result) {
			if (!($cnx->exec("update btw_$this->reference set bibkey='$newkey' where bibkey='$this->bibkey'") === FALSE)) $result += 1;
			if (!($cnx->exec("update btw_authors set bibkey='$newkey' where bibkey='$this->bibkey'") === FALSE)) $result += 1;
		}
		return $result;
    }	
	
	function update($title, $year, $month, $note, $scope, $abstract, $keywords, $tmpfile, $filename, $oldfile, $paperurl, $journalname, $volume, $number, $pages, $seriesname, $publishername, $edition, $organizationname, $booktitle, $url, $howpublished, $schoolname, $institutionname, $type) { // Save reference into DB
		$c = $this->config; require("db-connect.inc.php");
		// Upload file
		$file = $this->upload($tmpfile, $filename, $oldfile);		
		$q = "update btw_paper set title='" . addslashes($title) . "', yea=$year, mon='$month', note='" . addslashes($note) . "', scope='$scope', abstract='" . addslashes($abstract) . "', keywords='" . addslashes($keywords) . "', file='$file', url='$paperurl' where bibkey='$this->bibkey'";
		if ($cnx->exec($q) === FALSE) return 0;
		else return 1;
	}	
	
	function form_delete() {
		echo "<p>Delete reference $this->bibkey (<em>$this->title</em> - $this->reference)?</p>\n\n";
		echo "<div class=\"page\"><div class=\"column\"><form action=\"" . $_SERVER["PHP_SELF"] . "\" method=\"post\" enctype=\"multipart/form-data\"><fieldset>\n";
        echo "<input type=\"submit\" value=\"Cancel\" />\n";
        echo "</fieldset></form></div>\n<div class=\"column\">\n";
        echo "<form action=\"" . $_SERVER["PHP_SELF"] . "\" method=\"post\" enctype=\"multipart/form-data\"><fieldset>\n";
        echo "<input type=\"hidden\" name=\"reference\" value=\"$this->reference\" />\n";
        echo "<input type=\"hidden\" name=\"bibkey\" value=\"$this->bibkey\" />\n";
        echo "<input type=\"hidden\" NAME=\"action\" value=\"delete_ref\" />\n";
        echo "<input type=\"submit\" value=\"Delete\" />\n";
        echo "</fieldset></form></div></div>\n\n";	
	}
	
	function delete() { // Cascade delete
		$c = $this->config; require("db-connect.inc.php");
		if ($this->file) unlink("files/" . $this->file);
		$result = $cnx->exec("delete from btw_paper where bibkey='$this->bibkey'");
		if ($result) {
			$result2 += $cnx->exec("delete from btw_$this->reference where bibkey='$this->bibkey'");
			if ($result2) {
				$result += $result2;
				$result += $cnx->exec("delete from btw_authors where bibkey='$this->bibkey'");
			}
		}
		return $result;
	}
	
	function import($title, $year, $month, $note, $scope, $auth_id_list, $ed_id_list, $idseries, $volume, $number, $pages, $idinst, $edition, $idinstp, $booktitle, $idinsto, $howpublished, $type, $index_a, $index_e, $series, $publisher, $organization, $school, $institution, $abstract, $keywords) { // BibTeX import
		//echo "<p>index_a=$index_a index_e=$index_e</p>"; // Debug
		$c = $this->config; require("db-connect.inc.php");
		if ($abstract) $abins = "'" . addslashes($abstract) . "'"; else $abins = "NULL";
		if ($keywords) $kwins = "'" . addslashes($keywords) . "'"; else $kwins = "NULL";		
		$q = "insert into btw_paper values ('$this->bibkey', '$this->reference', '". addslashes($title) . "', $year, '$month', '". addslashes($note) ."', '$scope', $abins, $kwins, NULL, NULL)";
		if ($cnx->exec($q) == 0) return 0;
		else {
			// Authors
			$ida = explode(" ", $auth_id_list);  
			for ($i = 0; $i < $index_a; $i++) $result += $cnx->exec("insert into btw_authors values ('$this->bibkey', ". addslashes($ida[$i]) . ", $i+1)");
			$result += $index_e;
			return $result;
		}
	}
}


class Asi extends Cfg { // Superclass for Author, Series and Institution

	// Attributes
	
	public $id;
	public $name;
	
	protected $query; 
	protected $row; 
	
	// Methods
	
	function __construct($config, $id) { // Constructor
		parent::__construct($config);
		$this->id = $id;
		$this->name = "";
	}
	
	function load() { // Load Asi
		$c = $this->config; require("db-connect.inc.php");
		$q = "select * from btw_" . strtolower(get_class($this)) . " where id=$this->id";
		$result = $cnx->query($q);
		$this->row = $result->fetch(PDO::FETCH_ASSOC);
		$this->name = $this->row["name"];	
	}
	
	function save($name, $url = "", $address = "") { // Save Asi
		$this->query = "insert into btw_" . strtolower(get_class($this)) . " values($this->id, '" . addslashes($name) . "'";
	}	
	
	function update($name, $url = "", $address = "") { // Modify asi
		$this->query = "update btw_" . strtolower(get_class($this)) . " set name='" . addslashes($name) . "'";
	}	
	
	function delete() { // Delete asi
		$c = $this->config; require("db-connect.inc.php");
		return $cnx->exec("delete from btw_" . strtolower(get_class($this)) . " where id=$this->id");
	}	
	
	function form($action) { // Add-modify form for authors, series, and institutions
		echo "<form action=\"" . $_SERVER["PHP_SELF"] . "\" method=\"post\" enctype=\"multipart/form-data\"><fieldset>\n";
		echo "<input type=\"hidden\" name=\"action\" value=\"$action\" />\n";
		echo "<input type=\"hidden\" name=\"table\" value=\"" . strtolower(get_class($this)) . "\" />\n";
		echo "<h3>ID</h3><p>$this->id</p>\n";
		echo "<input type=\"hidden\" name=\"id\" value=\"$this->id\" />\n";
		echo "<h3>" . get_class($this) . " name</h3>\n";
		echo "<input type=\"text\" name=\"name\" size=\"50\" maxlength=\"100\" value=\"$this->name\" />\n";
	}
	
	protected function form_footer() {
		echo "<p><input type=\"reset\" name=\"Reset\" value=\"Reset\" />\n";
		echo "<input type=\"submit\" name=\"Submit\" value=\"Submit\" /></p>\n";
		echo "</fieldset></form>\n\n";
	}		
	
	function form_delete($c = 0) { // Delete form for authors, series, and institutions
		if ($c == 0) {
			$table = strtolower(get_class($this));
			echo "<p>Delete $table <em>$this->name</em>?</p>\n\n";
			echo "<div class=\"page\"><div class=\"column\"><form action=\"" . $_SERVER["PHP_SELF"] . "\" method=\"post\" enctype=\"multipart/form-data\"><fieldset>\n";
            echo "<input type=\"submit\" value=\"Cancel\" />\n";
            echo "</fieldset></form></div>\n\n";
            echo "<div class=\"column\"><form action=\"" . $_SERVER["PHP_SELF"] . "\" method=\"post\" enctype=\"multipart/form-data\"><fieldset>\n";
			echo "<input type=\"hidden\" name=\"table\" value=\"$table\" />\n";
            echo "<input type=\"hidden\" NAME=\"id\" value=\"$this->id\" />\n";
            echo "<input type=\"hidden\" NAME=\"action\" value=\"delete_asi\" />\n";
            echo "<input type=\"submit\" value=\"Delete\" />\n";
            echo "</fieldset></form></div></div>\n\n";
		} else echo "<p>Impossible to delete <em>$this->name</em>, it would violate database integrity.</p>\n\n";
	}
}

	
class Author extends Asi { // Author

	// Methods

	function __construct($config, $id) { // Constructor
		parent::__construct($config, $id);
	}
	
	function load() { // Load author
		parent::load();
	}
	
	function save($name, $url = "", $address = "") { // Save author
		parent::save($name);
		$this->query .= ")";
		$c = $this->config; require("db-connect.inc.php");		
		return $cnx->exec($this->query);
	}
	
	function update($name, $url = "", $address = "") { // Modify author
		parent::update($name);
		$this->query .= " where id=$this->id";
		$c = $this->config; require("db-connect.inc.php");		
		if ($cnx->exec($this->query) === FALSE) return 0;
		else return 1;
	}		
	
	function form($action) { // Add-modify form for authors
		parent::form($action);
		echo "<em>(First_Name Last_Name)</em>\n";
		parent::form_footer();
	}
	
	function form_delete($n = 0) { // Delete form for authors
		$c = $this->config; require("db-connect.inc.php");	
		$result = $cnx->query("select count(*) as c from btw_authors where id=$this->id");
		$row = $result->fetch(PDO::FETCH_ASSOC);		
		$n = $row["c"];
		$result = $cnx->query("select count(*) as c from btw_editors where id=$this->id");
		$row = $result->fetch(PDO::FETCH_ASSOC);		
		$n += $row["c"];		
		parent::form_delete($n);
		return $n;
	}
}


class Series extends Asi { // Journal or Series

	// Attributes

	public $url;

	// Methods

	function __construct($config, $id) { // Constructor
		parent::__construct($config, $id);
		$this->url = "";
	}
	
	function load() { // Load series
		parent::load();
		$this->url = $this->row["url"];
	}
	
	function save($name, $url = "", $address = "") { // Save series
		parent::save($name);
		$this->query .= ", '$url')";
		$c = $this->config; require("db-connect.inc.php");		
		return $cnx->exec($this->query);
	}
	
	function update($name, $url = "", $address = "") { // Modify series
		parent::update($name);
		$this->query .= ", url='$url' where id=$this->id";
		$c = $this->config; require("db-connect.inc.php");		
		if ($cnx->exec($this->query) === FALSE) return 0;
		else return 1;
	}	
	
	function form($action) { // Add-modify form for series
		parent::form($action);
		echo "<h3>Web page</h3>\n";
		echo "<input type=\"text\" name=\"url\" size=\"50\" maxlength=\"100\" value=\"$this->url\" /> <span class=\"small\">optional</span>\n";		
		parent::form_footer();
	}	
	
	function form_delete($n = 0) { // Delete form for series
		$c = $this->config; require("db-connect.inc.php");		
		$result = $cnx->query("select count(*) as c from btw_article where journal=$this->id");
		$row = $result->fetch(PDO::FETCH_ASSOC);
		$n = $row["c"];
		$result = $cnx->query("select count(*) as c from btw_book where series=$this->id");
		$row = $result->fetch(PDO::FETCH_ASSOC);
		$n += $row["c"];		
		$result = $cnx->query("select count(*) as c from btw_inbook where series=$this->id");
		$row = $result->fetch(PDO::FETCH_ASSOC);
		$n += $row["c"];		
		$result = $cnx->query("select count(*) as c from btw_inproceedings where series=$this->id");
		$row = $result->fetch(PDO::FETCH_ASSOC);
		$n += $row["c"];						
		parent::form_delete($n);
		return $n;		
	}	
}


class Institution extends Asi { // Publisher, School, Institution or Organization

	// Attributes

	public $address;
	public $url;

	// Methods

	function __construct($config, $id) { // Constructor
		parent::__construct($config, $id);
		$this->address = "";
		$this->url = "";
	}
	
	function load() { // Load institution
		parent::load();
        $this->address = $this->row["address"];
		$this->url = $this->row["url"];
	}
	
	function save($name, $url = "", $address = "") { // Save institution
		parent::save($name);
		$this->query .= ", '" . addslashes($address) . "', '$url')";
		$c = $this->config; require("db-connect.inc.php");		
		return $cnx->exec($this->query);
	}	
	
	function update($name, $url = "", $address = "") { // Modify institution
		parent::update($name);
		$this->query .= ", address='" . addslashes($address) . "', url='$url' where id=$this->id";
		$c = $this->config; require("db-connect.inc.php");		
		if ($cnx->exec($this->query) === FALSE) return 0;
		else return 1;
	}		
	
	function form($action) { // Add-modify form for institutions
		parent::form($action);
			echo "<h3>Address</h3>\n";
			echo "<input type=\"text\" name=\"address\" size=\"50\" maxlength=\"50\" value=\"$this->address\" /> <span class=\"small\">optional</span>\n";
			echo "<h3>Web page</h3>\n";
			echo "<input type=\"text\" name=\"url\" size=\"50\" maxlength=\"100\" value=\"$this->url\" /> <span class=\"small\">optional</span>\n";		
		parent::form_footer();
	}
	
	function form_delete($n = 0) { // Delete form for institutions	
		$c = $this->config; require("db-connect.inc.php");
		$result = $cnx->query("select count(*) as c from btw_book where publisher=$this->id");
		$row = $result->fetch(PDO::FETCH_ASSOC);
		$n = $row["c"];	
		$result = $cnx->query("select count(*) as c from btw_inbook where publisher=$this->id");
		$row = $result->fetch(PDO::FETCH_ASSOC);
		$n += $row["c"];	
		$result = $cnx->query("select count(*) as c from btw_inproceedings where publisher=$this->id");
		$row = $result->fetch(PDO::FETCH_ASSOC);
		$n += $row["c"];	
		$result = $cnx->query("select count(*) as c from btw_phdthesis where school=$this->id");
		$row = $result->fetch(PDO::FETCH_ASSOC);
		$n += $row["c"];	
		$result = $cnx->query("select count(*) as c from btw_techreport where institution=$this->id");
		$row = $result->fetch(PDO::FETCH_ASSOC);
		$n += $row["c"];	
		parent::form_delete($n);
		return $n;		
	}	
}


class Article extends Paper { // Journal paper

	// Attributes

	public $volume;
	public $number;
	public $pages;

	// Associations

	public $journal;

	// Methods

	function __construct($config, $bibkey) { // Constructor
		// Common attributes
		parent::__construct($config, $bibkey);
		// Specific attributes
		$this->volume = "";
		$this->number = "";
		$this->pages = "";
	}
	
	function load() { // Load article
		// Load common attributes
		parent::load();
		// Load specific attributes
		$row = parent::load_specific_attributes();
		$this->journal = new Series($this->config, $row["journal"]);
		$this->journal->load();
		$this->number = $row["number"];
		$this->volume = $row["volume"];
		$this->pages = $row["pages"];
		// BibTeX & XML computation
		$this->bibtex .= "&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;Journal = {" . $this->journal->name . "},<br />\n";
		$this->xml .= "&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<journal>" . $this->journal->name . "</journal><br />\n";
		if ($this->volume) {
			$this->bibtex .= "&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;Volume = {" . $this->volume . "},<br />\n";
			$this->xml .= "&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<volume>$this->volume</volume><br />\n";
		}
		if ($this->number) {
			$this->bibtex .= "&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;Number = {" . $this->number . "},<br />\n";
			$this->xml .= "&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<number>$this->number</number><br />\n";
		}
		if ($this->month) {
			$this->bibtex .= "&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;Month = {" . $this->month . "},<br />\n";
			$this->xml .= "&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<month>$this->month</month><br />\n";
		}
		$this->bibtex .= "&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;Year = {" . $this->year . "}";
		$this->xml .= "&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<year>" . $this->year . "</year><br />\n";
		if ($this->pages) {
			$this->bibtex .= ",<br />\n&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;Pages = {" . $this->pages . "}";
			$this->xml .= "&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<pages>" . $this->pages . "</pages><br />\n";
		}
		parent::bibtex_xml_footer();
	}

	function display($cutnpaste = "off") {
		$d = parent::display() . ", ";
    	if (($this->journal->url) && ($cutnpaste != "on")) $d .= "<a href=\"" . $this->journal->url. "\">";
    	$d .= "<em>" . $this->journal->name . "</em>";
    	if (($this->journal->url) && ($cutnpaste != "on")) $d .= "</a>";
    	if ($this->volume) $d .= ", Vol. $this->volume";
    	if ($this->number) $d .= ", No. $this->number";
	   $d .= ", ";
    	if ($this->month) $d .= "$this->month ";
    	$d .= $this->year;
    	if ($this->pages) $d .= ", $this->pages";
		return $d . parent::display_footer();
	}

	function header() {
		return "$this->scope journals";
	}
	
	function form($action) { // Add-modify form: specific fields
		parent::form($action);
	    echo "<h3>Journal</h3>\n";
		$t = new Table($this->config, "series");
		$t->select_name("journal", $this->journal->id);
		echo "<h3>Volume</h3>\n<input type=\"text\" name=\"volume\" size=\"10\" maxlength=\"10\" value=\"$this->volume\" /> <span class=\"small\">optional</span>\n";
		echo "<h3>Number</h3>\n<input type=\"text\" name=\"number\" size=\"10\" maxlength=\"10\" value=\"$this->number\" /> <span class=\"small\">optional</span>\n";
		echo "<h3>Pages</h3>\n<input type=\"text\" name=\"pages\" size=\"15\" maxlength=\"15\" value=\"$this->pages\" /> <span class=\"small\">optional</span>\n";
		parent::form_footer();
	}
	
	function save($title, $year, $month, $note, $scope, $abstract, $keywords, $tmpfile, $filename, $oldfile, $paperurl, $journalname, $volume, $number, $pages, $seriesname, $publishername, $edition, $organizationname, $booktitle, $url, $howpublished, $schoolname, $institutionname, $type) {
		if (parent::save($title, $year, $month, $note, $scope, $abstract, $keywords, $tmpfile, $filename, $oldfile, $paperurl, $journalname, $volume, $number, $pages, $seriesname, $publishername, $edition, $organizationname, $booktitle, $url, $howpublished, $schoolname, $institutionname, $type)) {
			$t = new Table($this->config, "series");
			$journalid = $t->get_id($journalname);
			$c = $this->config; require("db-connect.inc.php");
			return $cnx->exec("insert into btw_article values ('$this->bibkey', $journalid, '$volume', '$number', '$pages')");
		} else return 0;
	}
	
	function update($title, $year, $month, $note, $scope, $abstract, $keywords, $tmpfile, $filename, $oldfile, $paperurl, $journalname, $volume, $number, $pages, $seriesname, $publishername, $edition, $organizationname, $booktitle, $url, $howpublished, $schoolname, $institutionname, $type) {
		if (parent::update($title, $year, $month, $note, $scope, $abstract, $keywords, $tmpfile, $filename, $oldfile, $paperurl, $journalname, $volume, $number, $pages, $seriesname, $publishername, $edition, $organizationname, $booktitle, $url, $howpublished, $schoolname, $institutionname, $type)) {
			$t = new Table($this->config, "series");
			$journalid = $t->get_id($journalname);	
			$c = $this->config; require("db-connect.inc.php");
			$q = "update btw_article set journal=$journalid, volume='$volume', number='$number', pages='$pages' where bibkey='$this->bibkey'";
			if ($cnx->exec($q) === FALSE) return 0;
			else return 1;			
		} else return 0;
	}	
	
	function import($title, $year, $month, $note, $scope, $auth_id_list, $ed_id_list, $idseries, $volume, $number, $pages, $idinst, $edition, $idinstp, $booktitle, $idinsto, $howpublished, $type, $index_a, $index_e, $series, $publisher, $organization, $school, $institution, $abstract, $keywords) { // BibTeX import
		if (parent::import($title, $year, $month, $note, $scope, $auth_id_list, $ed_id_list, $idseries, $volume, $number, $pages, $idinst, $edition, $idinstp, $booktitle, $idinsto, $howpublished, $type, $index_a, $index_e, $series, $publisher, $organization, $school, $institution, $abstract, $keywords)) {
			$c = $this->config; require("db-connect.inc.php");
			return $cnx->exec("insert into btw_article values ('$this->bibkey', $idseries, '$volume', '$number', '$pages')");
		} else 
			return 0;
	}
}


class Book extends Paper { // Book

	// Attributes

	public $volume;
	public $edition;

	// Associations

	public $editors;
	public $series;
	public $publisher;

	// Methods
	
	function __construct($config, $bibkey) { // Constructor
		// Common attributes
		parent::__construct($config, $bibkey);
		// Specific attributes
		$this->volume = "";
		$this->edition = "";
	}	

	function load() { // Load book
		// Load common attributes
		parent::load();
		// Load editors
		$this->editors = array(); 
		$i = 0;
		$q = "select name from btw_author a, btw_book_editors b where a.id=b.id and bibkey='$this->bibkey' order by rank";	
		$c = $this->config; require("db-connect.inc.php");
		foreach($cnx->query($q) as $row) $this->editors[$i++] = $row["name"];	
		// Load specific attributes
		$row = parent::load_specific_attributes();
		$this->publisher = New Institution($this->config, $row["publisher"]);
		$this->publisher->load();
		if ($row["series"]) {
			$this->series = New Series($this->config, $row["series"]);
			$this->series->load();
		}
		$this->volume = $row["volume"];
		$this->edition = $row["edition"];
		// BibTeX & XML computation
		if (count($this->editors)) {
	      $this->bibtex .= "&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;Editor = {";
	      $this->xml .= "&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<editor>";
			for ($i = 0; $i < count($this->editors); $i++) {
			  if ($i > 0) {
			    $this->bibtex .= " and ";
		   	 $this->xml .= ", ";
			  }
			  $this->bibtex .= $this->editors[$i];
			  $this->xml .= $this->editors[$i];  
			}
	      $this->bibtex.="},<br />\n";
		   $this->xml.="</editor><br />\n";
		}		
		if ($this->edition) {
	      $this->bibtex .= "&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;Edition = {" . $this->edition . "},<br />\n";
	      $this->xml .= "&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<edition>" . $this->edition . "</edition><br />\n";
		}
		if ($row["series"]) {
			$this->bibtex .= "&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;Series = {" . $this->series->name . "},<br />\n";
			$this->xml .= "&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<series>" . $this->series->name . "</series><br />\n";  
		}
    	if ($this->volume) {
			$this->bibtex .= "&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;Volume = {" . $this->volume . "},<br />\n";
			$this->xml .= "&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<volume>" . $this->volume. "</volume><br />\n";
		}
    	$this->bibtex .= "&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;Publisher = {" . $this->publisher->name . "},<br />\n";
		$this->xml .= "&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<publisher>" . $this->publisher->name . "</publisher><br />\n";
    	if ($this->publisher->address) {
    		$this->bibtex .= "&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;Address = {" . $this->publisher->address . "},<br />\n";
    		$this->xml .= "&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<address>" . $this->publisher->address . "</address><br />\n";
		}	
    	if ($this->month) {
			$this->bibtex .= "&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;Month = {" . $this->month . "},<br />\n";
			$this->xml .= "&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<month>" . $this->month . "</month><br />\n";
    	}
    	$this->bibtex .= "&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;Year = {" . $this->year . "}";
    	$this->xml .= "&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<year>" . $this->year. "</year><br />\n";
		parent::bibtex_xml_footer();
	}

	function display($cutnpaste = "off") {
		$d = parent::display();
		if (count($this->editors)) {
			for ($i = 0; $i < count($this->editors); $i++) {
				if ($i > 0) $d .= ", ";
				$namecomponents = explode(" ", $this->editors[$i]);
				$namecomponents2 = explode("-", $this->editors[$i]);
				$initial = strtoupper(substr($namecomponents[0], 0, 1)) . ".";
				$lastname = ucfirst($namecomponents[count($namecomponents)-1]);
				if (count($namecomponents) > 2) $secondinitial = strtoupper(substr($namecomponents[1], 0, 1)) . ".";
				elseif (count($namecomponents2) > 1) $secondinitial = strtoupper(substr($namecomponents2[1], 0, 1)) . ".";
				else $secondinitial = "";
				$d .= "$initial$secondinitial $lastname"; 
			}
			if (count($this->editors) > 1) $d .= ", Eds.";
			else $d .= ", Ed.";
			$d .= ", <em>$this->title</em>";
		}		
   	if ($this->edition) $d .= ", $this->edition";
    	if ($this->series) {
	      $d .= ", ";
	      if (($this->series->url) && ($cutnpaste != "on")) $d .= "<a href=\"" . $this->series->url . "\">";
      	$d .= "<em>" . $this->series->name . "</em>";
      	if (($this->series->url) && ($cutnpaste != "on")) $d .= "</a>";
    	}
    	if ($this->volume) $d .= ", Vol. $this->volume";
    	$d .= ", ";
    	if (($this->publisher->url) && ($cutnpaste != "on")) $d .= "<a href=\"" . $this->publisher->url . "\">";
    	$d .= $this->publisher->name;
    	if (($this->publisher->url) && ($cutnpaste != "on")) $d .= "</a>";
    	$d .= ", ";
    	if ($this->publisher->address) $d .= $this->publisher->address .", ";
    	if ($this->month) $d .= $this->month . " ";
		$d .= $this->year;
		return $d . parent::display_footer();
	}

	function header() {
		return "Books of " . strtolower($this->scope) . " scope";
	}
	
	function form($action) { // Add-modify form
		parent::form($action);
		// Editors
		$c = $this->config; require("db-connect.inc.php");
		$q = "select name from btw_author as a, btw_book_editors as b where a.id=b.id and bibkey='$this->bibkey'";
		$rowcount = $cnx->query($q);
		$n = count($rowcount->fetchAll());
		if ($n) {
			$result = $cnx->query($q);
			echo "<h3>Editor(s)</h3>\n<ul>\n";
			foreach($result as $row) echo "<li>" . $row["name"] . "</li>\n";
			echo "</ul>\n";
		}		
		echo "<h3>Series</h3>\n";
		$t = new Table($this->config, "series");
		$t->select_name("series", $this->series->id, 1);
		echo "<h3>Volume</h3>\n<input type=\"text\" name=\"volume\" size=\"10\" maxlength=\"10\" value=\"$this->volume\" /> <span class=\"small\">optional</span>\n";
		echo "<h3>Edition</h3>\n<input type=\"text\" name=\"edition\" size=\"15\" maxlength=\"15\" value=\"$this->edition\" /> <span class=\"small\">optional</span>\n";
		echo "<h3>Publisher</h3>\n";
		$t = new Table($this->config, "institution");
		$t->select_name("publisher", $this->publisher->id);
		parent::form_footer();
	}
	
	function save($title, $year, $month, $note, $scope, $abstract, $keywords, $tmpfile, $filename, $oldfile, $paperurl, $journalname, $volume, $number, $pages, $seriesname, $publishername, $edition, $organizationname, $booktitle, $url, $howpublished, $schoolname, $institutionname, $type) {
		if (parent::save($title, $year, $month, $note, $scope, $abstract, $keywords, $tmpfile, $filename, $oldfile, $paperurl, $journalname, $volume, $number, $pages, $seriesname, $publishername, $edition, $organizationname, $booktitle, $url, $howpublished, $schoolname, $institutionname, $type)) {	
			if ($seriesname) {
				$t = new Table($this->config, "series");
				$seriesid = $t->get_id($seriesname);
			} else $seriesid = "NULL";
			$t = new Table($this->config, "institution");
			$publisherid = $t->get_id($publishername);
			$c = $this->config; require("db-connect.inc.php");
			return $cnx->exec("insert into btw_book values ('$this->bibkey', $seriesid, $publisherid, '$volume', '$edition')");
		} else return 0;
	}	
	
	function update($title, $year, $month, $note, $scope, $abstract, $keywords, $tmpfile, $filename, $oldfile, $paperurl, $journalname, $volume, $number, $pages, $seriesname, $publishername, $edition, $organizationname, $booktitle, $url, $howpublished, $schoolname, $institutionname, $type) {
		if (parent::update($title, $year, $month, $note, $scope, $abstract, $keywords, $tmpfile, $filename, $oldfile, $paperurl, $journalname, $volume, $number, $pages, $seriesname, $publishername, $edition, $organizationname, $booktitle, $url, $howpublished, $schoolname, $institutionname, $type)) {	
			if ($seriesname) {
				$t = new Table($this->config, "series");
				$seriesid = $t->get_id($seriesname);
			} else $seriesid = "NULL";
			$t = new Table($this->config, "institution");
			$publisherid = $t->get_id($publishername);
			$c = $this->config; require("db-connect.inc.php");
			$q = "update btw_book set series=$seriesid, publisher=$publisherid, volume='$volume', edition='$edition' where bibkey='$this->bibkey'";
			if ($cnx->exec($q) === FALSE) return 0;
			else return 1;
		} else return 0;
	}	
	
	function delete() {
		$result = parent::delete();
		if ($result) {
			$c = $this->config; require("db-connect.inc.php");
			$result += $cnx->exec("delete from btw_book_editors where bibkey='$this->bibkey'");
		}
		return $result;
	}
		
	function import($title, $year, $month, $note, $scope, $auth_id_list, $ed_id_list, $idseries, $volume, $number, $pages, $idinst, $edition, $idinstp, $booktitle, $idinsto, $howpublished, $type, $index_a, $index_e, $series, $publisher, $organization, $school, $institution, $abstract, $keywords) { // BibTeX import
		if (parent::import($title, $year, $month, $note, $scope, $auth_id_list, $ed_id_list, $idseries, $volume, $number, $pages, $idinst, $edition, $idinstp, $booktitle, $idinsto, $howpublished, $type, $index_a, $index_e, $series, $publisher, $organization, $school, $institution, $abstract, $keywords)) {
			$c = $this->config; require("db-connect.inc.php");
			// Editors
			$ide = explode(" ", $ed_id_list);  
			for ($i = 0; $i < $index_e; $i++) 		
				$result += $cnx->exec("insert into btw_book_editors values ('$this->bibkey', ". addslashes($ide[$i]) . ", $i+1)");	
			if (!$series) $idseries = "NULL";
			$result += $cnx->exec("insert into btw_book values ('$this->bibkey', $idseries, $idinst, '$volume', '" . addslashes($edition) . "')");
			return $result;
		} else return 0;
	}	
}


class Inbook extends Paper { // Book chapter

	// Attributes

	public $pages;
	public $volume;
	public $edition;

	// Associations

	public $series;
	public $publisher;

	// Methods
	
	function __construct($config, $bibkey) { // Constructor
		// Common attributes
		parent::__construct($config, $bibkey);
		$this->dblpref = "incollection"; // DBLP's DTD doesn't know some BibTeX entries, unfortunately...
		// Specific attributes
		$this->volume = "";
		$this->edition = "";
		$this->pages = "";
	}	

	function load() { // Load inbook
		// Load common attributes
		parent::load();
		// Load specific attributes
		$row = parent::load_specific_attributes();
		$this->publisher = New Institution($this->config, $row["publisher"]);
		$this->publisher->load();
		if ($row["series"]) {
			$this->series = New Series($this->config, $row["series"]);
			$this->series->load();
		}
		$this->pages = $row["pages"];
		$this->volume = $row["volume"];
		$this->edition = $row["edition"];
		// BibTeX & XML computation
		if ($row["series"]) {
			$this->bibtex .= "&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;Series = {" . $this->series->name . "},<br />\n";
			$this->xml .= "&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<series>" . $this->series->name . "</series><br />\n";  
		}
    	if ($this->volume) {
			$this->bibtex .= "&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;Volume = {" . $this->volume . "},<br />\n";
			$this->xml .= "&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<volume>" . $this->volume . "</volume><br />\n";
		}
		if ($this->edition) {
	      $this->bibtex .= "&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;Edition = {" . $this->edition . "},<br />\n";
	      $this->xml .= "&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<edition>" . $this->edition. "</edition><br />\n";
		}
    	$this->bibtex .= "&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;Publisher = {" . $this->publisher->name . "},<br />\n";
		$this->xml .= "&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<publisher>" . $this->publisher->name . "</publisher><br />\n";
    	if ($this->publisher->address) {
    		$this->bibtex .= "&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;Address = {" . $this->publisher->address . "},<br />\n";
    		$this->xml .= "&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<address>" . $this->publisher->address . "</address><br />\n";
		}	
    	if ($this->month) {
			$this->bibtex .= "&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;Month = {" . $this->month . "},<br />\n";
			$this->xml .= "&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<month>" . $this->month . "</month><br />\n";
    	}
    	$this->bibtex .= "&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;Year = {" . $this->year . "}";
    	$this->xml .= "&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<year>" . $this->year . "</year><br />\n";
		if ($this->pages) {
	    	$this->bibtex .= ",<br />\n&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;Pages = {" . $this->pages . "}";
			$this->xml .= "&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<pages>" . $this->pages . "</pages><br />\n";
		}
		parent::bibtex_xml_footer();
	}

	function display($cutnpaste = "off") {
		$d = parent::display();
    	if ($this->series) {
	      $d .= ", ";
	      if (($this->series->url) && ($cutnpaste != "on")) $d .= "<a href=\"" . $this->series->url . "\">";
      	$d .= "<em>" . $this->series->name . "</em>";
      	if (($this->series->url) && ($cutnpaste != "on")) $d .= "</a>";
    	}
    	if ($this->volume) $d .= ", Vol. $this->volume";
   	if ($this->edition) $d .= ", $this->edition";
    	$d .= ", ";
    	if (($this->publisher->url) && ($cutnpaste != "on")) $d .= "<a href=\"" . $this->publisher->url . "\">";
    	$d .= $this->publisher->name;
    	if (($this->publisher->url) && ($cutnpaste != "on")) $d .= "</a>";
    	$d .= ", ";
    	if ($this->publisher->address) $d .= $this->publisher->address . ", ";
    	if ($this->month) $d .= $this->month . " ";
		$d .= $this->year;
		if ($this->pages) $d .= ", $this->pages";
		return $d . parent::display_footer();
	}

	function header() {
		return "Book chapters (" . strtolower($this->scope) . " scope)";
	}
	
	function form($action) { // Add-modify form
		parent::form($action);
		echo "<h3>Series</h3>\n";
		$t = new Table($this->config, "series");
		$t->select_name("series", $this->series->id, 1);
		echo "<h3>Volume</h3>\n<input type=\"text\" name=\"volume\" size=\"10\" maxlength=\"10\" value=\"$this->volume\" /> <span class=\"small\">optional</span>\n";
		echo "<h3>Edition</h3>\n<input type=\"text\" name=\"edition\" size=\"15\" maxlength=\"15\" value=\"$this->edition\" /> <span class=\"small\">optional</span>\n";
		echo "<h3>Publisher</h3>\n";
		$t = new Table($this->config, "institution");
		$t->select_name("publisher", $this->publisher->id);
		echo "<h3>Pages</h3>\n<input type=\"text\" name=\"pages\" size=\"15\" maxlength=\"15\" value=\"$this->pages\" /> <span class=\"small\">optional</span>\n";
		parent::form_footer();
	}
	
	function save($title, $year, $month, $note, $scope, $abstract, $keywords, $tmpfile, $filename, $oldfile, $paperurl, $journalname, $volume, $number, $pages, $seriesname, $publishername, $edition, $organizationname, $booktitle, $url, $howpublished, $schoolname, $institutionname, $type) {
		if (parent::save($title, $year, $month, $note, $scope, $abstract, $keywords, $tmpfile, $filename, $oldfile, $paperurl, $journalname, $volume, $number, $pages, $seriesname, $publishername, $edition, $organizationname, $booktitle, $url, $howpublished, $schoolname, $institutionname, $type)) {	
			if ($seriesname) {
				$t = new Table($this->config, "series");
				$seriesid = $t->get_id($seriesname);
			} else $seriesid = "NULL";
			$t = new Table($this->config, "institution");
			$publisherid = $t->get_id($publishername);
			$c = $this->config; require("db-connect.inc.php");
			return $cnx->exec("insert into btw_inbook values ('$this->bibkey', $seriesid, $publisherid, '$volume', '$pages', '$edition')");
		} else return 0;
	}	

	function update($title, $year, $month, $note, $scope, $abstract, $keywords, $tmpfile, $filename, $oldfile, $paperurl, $journalname, $volume, $number, $pages, $seriesname, $publishername, $edition, $organizationname, $booktitle, $url, $howpublished, $schoolname, $institutionname, $type) {
		if (parent::update($title, $year, $month, $note, $scope, $abstract, $keywords, $tmpfile, $filename, $oldfile, $paperurl, $journalname, $volume, $number, $pages, $seriesname, $publishername, $edition, $organizationname, $booktitle, $url, $howpublished, $schoolname, $institutionname, $type)) {	
			if ($seriesname) {
				$t = new Table($this->config, "series");
				$seriesid = $t->get_id($seriesname);
			} else $seriesid = "NULL";
			$t = new Table($this->config, "institution");
			$publisherid = $t->get_id($publishername);
			$c = $this->config; require("db-connect.inc.php");
			$q = "update btw_inbook set series=$seriesid, publisher=$publisherid, volume='$volume', pages='$pages', edition='$edition' where bibkey='$this->bibkey'";
			if ($cnx->exec($q) === FALSE) return 0;
			else return 1;
		} else return 0;
	}		
	
	function import($title, $year, $month, $note, $scope, $auth_id_list, $ed_id_list, $idseries, $volume, $number, $pages, $idinst, $edition, $idinstp, $booktitle, $idinsto, $howpublished, $type, $index_a, $index_e, $series, $publisher, $organization, $school, $institution, $abstract, $keywords) { // BibTeX import
		if (parent::import($title, $year, $month, $note, $scope, $auth_id_list, $ed_id_list, $idseries, $volume, $number, $pages, $idinst, $edition, $idinstp, $booktitle, $idinsto, $howpublished, $type, $index_a, $index_e, $series, $publisher, $organization, $school, $institution, $abstract, $keywords)) {
			if (!$series) $idseries = "NULL";
			if (!$publisher) $idinst = "NULL";
			$c = $this->config; require("db-connect.inc.php");
			return $cnx->exec("insert into btw_inbook values ('$this->bibkey', $idseries, $idinst, '$volume', '$pages', '$edition')");			
		} else return 0;
	}	
}


class Proceedings extends Paper { // Conference proceedings

	// Attributes
	
	public $volume;
	public $url;
	
	// Associations
	
	public $editors;
	public $organization;
	public $series;
	public $publisher;
	
	// Methods
	
	function __construct($config, $bibkey) { // Constructor
		// Common attributes
		parent::__construct($config, $bibkey);
		// Specific attributes
		$this->volume = "";
		$this->url = "";
	}	
	
	function load() { // Load proceedings
		// Load common attributes
		parent::load();
		// Load editors
		$this->editors = array(); 
		$i = 0;
		$q = "select name from btw_author a, btw_proc_editors b where a.id=b.id and bibkey='$this->bibkey' order by rank";
		$c = $this->config; require("db-connect.inc.php");
		$result = $cnx->query($q);		
		foreach ($result as $row) $this->editors[$i++] = $row["name"];
		// Load specific attributes
		$row = parent::load_specific_attributes();
		if ($row["publisher"]) {
			$this->publisher = New Institution($this->config, $row["publisher"]);
			$this->publisher->load();
		}
		if ($row["series"]) {
			$this->series = New Series($this->config, $row["series"]);
			$this->series->load();
		}
		$this->volume = $row["volume"];
		if ($row["organization"]) {
			$this->organization = New Institution($this->config, $row["organization"]);
			$this->organization->load();
		}
		$this->url = $row["url"];
		// BibTeX & XML computation
		if (count($this->editors)) {
	      $this->bibtex .= "&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;Editor = {";
	      $this->xml .= "&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<editor>";
			for ($i = 0; $i < count($this->editors); $i++) {
			  if ($i > 0) {
			    $this->bibtex .= " and ";
		   	 $this->xml .= ", ";
			  }
			  $this->bibtex .= $this->editors[$i];
			  $this->xml .= $this->editors[$i];  
			}
	      $this->bibtex.="},<br />\n";
		  $this->xml.="</editor><br />\n";
		}
    	if ($this->month) {
			$this->bibtex .= "&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;Month = {" . $this->month . "},<br />\n";
			$this->xml .= "&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<month>" . $this->month . "</month><br />\n";
    	}
    	$this->bibtex .= "&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;Year = {" . $this->year . "}";
    	$this->xml .= "&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<year>" . $this->year . "</year>";
		if ($row["series"]) {
			$this->bibtex .= ",<br />\n&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;Series = {" . $this->series->name . "}";
			$this->xml .= "<br />\n&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<series>" . $this->series->name . "</series><br />\n";  
    		if ($this->volume) {
				$this->bibtex .= ",<br />\n&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;Volume = {" . $this->volume . "}";
				$this->xml .= "&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<volume>" . $this->volume . "</volume><br />\n";
			}
		}
    	if ($row["organization"]) {
    		$this->bibtex .= ",<br />\n&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;Organization = {".$this->organization->name."}";
    		$this->xml .= "&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<publisher>" . $this->organization->name . "</publisher><br />\n";
    	}	
		if ($row["publisher"]) {
    		$this->bibtex .= ",<br />\n&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;Publisher = {" . $this->publisher->name . "}";
			$this->xml .= "&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<publisher>" . $this->publisher->name . "</publisher><br />\n";
    		if ($this->publisher->address) {
    			$this->bibtex .= ",<br />\n&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;Address = {" . $this->publisher->address . "}";
    			$this->xml .= "&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<address>" . $this->publisher->address . "</address><br />\n";
			}	
		}
		parent::bibtex_xml_footer();
	}	
	
	function display($cutnpaste = "off") {
		$d = parent::display();
		if (count($this->editors)) {
			for ($i = 0; $i < count($this->editors); $i++) {
				if ($i > 0) $d .= ", ";
				$namecomponents = explode(" ", $this->editors[$i]);
				$namecomponents2 = explode("-", $this->editors[$i]);
				$initial = strtoupper(substr($namecomponents[0], 0, 1)) . ".";
				$lastname = ucfirst($namecomponents[count($namecomponents)-1]);
				if (count($namecomponents) > 2) $secondinitial = strtoupper(substr($namecomponents[1], 0, 1)) . ".";
				elseif (count($namecomponents2) > 1) $secondinitial = strtoupper(substr($namecomponents2[1], 0, 1)) . ".";
				else $secondinitial = "";
				$d .= "$initial$secondinitial $lastname"; 				  
			}
			if (count($this->editors) > 1) $d .= ", Eds., ";
			else $d .= ", Ed., ";
		}
		$d .= "<em>";
		if (($this->url) && ($cutnpaste == "off")) $d .= "<a href=\"$this->url\">";
		$d .= $this->title;
		if (($this->url) && ($cutnpaste == "off")) $d .= "</a>";
		$d .= "</em>, ";
    	if ($this->month) $d .= $this->month . " ";
		$d .= $this->year;
    	if ($this->series) {
	      $d .= ", ";
	      if (($this->series->url) && ($cutnpaste != "on")) $d .= "<a href=\"" . $this->series->url . "\">";
      	$d .= "<em>" . $this->series->name . "</em>";
      	if (($this->series->url) && ($cutnpaste != "on")) $d .= "</a>";
			if ($this->volume) $d .= ", Vol. $this->volume";
			if ($this->publisher) {
				$d .= ", ";
		  		if (($this->publisher->url) && ($cutnpaste != "on")) $d .= "<a href=\"" . $this->publisher->url . "\">";
		  		$d .= $this->publisher->name;
			  	if (($this->publisher->url) && ($cutnpaste != "on")) $d .= "</a>";
				if ($this->publisher->address) $d .= ", " . $this->publisher->address;		    	
			}
		} else {		
			if ($this->publisher) {
				$d .= "; ";
		  		if (($this->publisher->url) && ($cutnpaste != "on")) $d .= "<a href=\"" . $this->publisher->url . "\">";
		  		$d .= $this->publisher->name;
			  	if (($this->publisher->url) && ($cutnpaste != "on")) $d .= "</a>";
				if ($this->publisher->address) $d .= ", " . $this->publisher->address;		    	
			}    	
			if ($this->organization) {
				$d .= "; ";
		  		if (($this->organization->url) && ($cutnpaste != "on")) $d .= "<a href=\"" . $this->organization->url . "\">";
		  		$d .= $this->organization->name;
			  	if (($this->organization->url) && ($cutnpaste != "on")) $d .= "</a>";
				if ($this->organization->address) $d .= ", " . $this->organization->address;		    	
			} 			
    	}
		return $d . parent::display_footer();
	}

	function header() {
		return $this->scope . " conference proceedings";
	}	
	
	function form($action) { // Add-modify form
		parent::form($action);
		// Editors
		echo "<h3>Editor(s)</h3>\n<ul>\n";		
		$c = $this->config; require("db-connect.inc.php");
		$result = $cnx->query("select name from btw_author as a, btw_proc_editors as b where a.id=b.id and bibkey='$this->bibkey'");		
		foreach ($result as $row) echo "<li>" . $row["name"] . "</li>\n";
		echo "</ul>\n";
		echo "<h3>Organization</h3>\n";
		$t = new Table($this->config, "institution");
		$t->select_name("organization", $this->organization->id, 1);
		echo "<h3>Series</h3>\n";
		$t = new Table($this->config, "series");
		$t->select_name("series", $this->series->id, 1);
		echo "<h3>Volume</h3>\n<input type=\"text\" name=\"volume\" size=\"10\" maxlength=\"10\" value=\"$this->volume\" /> <span class=\"small\">optional</span>\n";
		echo "<h3>Publisher</h3>\n";
		$t = new Table($this->config, "institution");
		$t->select_name("publisher", $this->publisher->id, 1);
		echo "<h3>Conference web page</h3>\n<input type=\"text\" name=\"url\" size=\"50\" maxlength=\"100\" value=\"$this->url\" /> <span class=\"small\">optional</span>\n";
		parent::form_footer();
	}	
	
	function save($title, $year, $month, $note, $scope, $abstract, $keywords, $tmpfile, $filename, $oldfile, $paperurl, $journalname, $volume, $number, $pages, $seriesname, $publishername, $edition, $organizationname, $booktitle, $url, $howpublished, $schoolname, $institutionname, $type) {
		if (parent::save($title, $year, $month, $note, $scope, $abstract, $keywords, $tmpfile, $filename, $oldfile, $paperurl, $journalname, $volume, $number, $pages, $seriesname, $publishername, $edition, $organizationname, $booktitle, $url, $howpublished, $schoolname, $institutionname, $type)) {	
			if ($seriesname) {
				$t = new Table($this->config, "series");
				$seriesid = $t->get_id($seriesname);
			} else $seriesid = "NULL";
			if ($publishername) {
				$t = new Table($this->config, "institution");
				$publisherid = $t->get_id($publishername);
			} else $publisherid = "NULL";
			if ($organizationname) {
				$t = new Table($this->config, "institution");
				$organizationid = $t->get_id($organizationname);
			} else $organizationid = "NULL";		
			$c = $this->config; require("db-connect.inc.php");
			return $cnx->exec("insert into btw_proceedings values ('$this->bibkey', '$volume', '$url', $seriesid, $publisherid, $organizationid)");
		} else return 0;
	}	
	
	function update($title, $year, $month, $note, $scope, $abstract, $keywords, $tmpfile, $filename, $oldfile, $paperurl, $journalname, $volume, $number, $pages, $seriesname, $publishername, $edition, $organizationname, $booktitle, $url, $howpublished, $schoolname, $institutionname, $type) {
		if (parent::update($title, $year, $month, $note, $scope, $abstract, $keywords, $tmpfile, $filename, $oldfile, $paperurl, $journalname, $volume, $number, $pages, $seriesname, $publishername, $edition, $organizationname, $booktitle, $url, $howpublished, $schoolname, $institutionname, $type)) {	
			if ($seriesname) {
				$t = new Table($this->config, "series");
				$seriesid = $t->get_id($seriesname);
			} else $seriesid = "NULL";
			if ($publishername) {
				$t = new Table($this->config, "institution");
				$publisherid = $t->get_id($publishername);
			} else $publisherid = "NULL";
			if ($organizationname) {
				$t = new Table($this->config, "institution");
				$organizationid = $t->get_id($organizationname);
			} else $organizationid = "NULL";
			$c = $this->config; require("db-connect.inc.php");
			$q = "update btw_proceedings set series=$seriesid, publisher=$publisherid, volume='$volume', organization=$organizationid, url='$url' where bibkey='$this->bibkey'";	
			if ($cnx->exec($q) === FALSE) return 0;
			else return 1;
		} else return 0;
	}	
	
	function delete() {
		$result = parent::delete();
		$c = $this->config; require("db-connect.inc.php");
		if ($result) $result += $cnx->exec("delete from btw_proc_editors where bibkey='$this->bibkey'");
		return $result;
	}
	
	function import($title, $year, $month, $note, $scope, $auth_id_list, $ed_id_list, $idseries, $volume, $number, $pages, $idinst, $edition, $idinstp, $booktitle, $idinsto, $howpublished, $type, $index_a, $index_e, $series, $publisher, $organization, $school, $institution, $abstract, $keywords) { // BibTeX import
		if (parent::import($title, $year, $month, $note, $scope, $auth_id_list, $ed_id_list, $idseries, $volume, $number, $pages, $idinst, $edition, $idinstp, $booktitle, $idinsto, $howpublished, $type, $index_a, $index_e, $series, $publisher, $organization, $school, $institution, $abstract, $keywords)) {
			// Editors
			$c = $this->config; require("db-connect.inc.php");
			//echo "<p>editors=$ed_id_list</p>"; // Debug
			$ide = explode(" ", $ed_id_list); 			
			for ($i = 0; $i < $index_e; $i++) {
				$q = "insert into btw_proc_editors values ('$this->bibkey', ". addslashes($ide[$i]) . ", $i+1)";
				//echo "<p>$q</p>"; // Debug
				$result += $cnx->exec($q);
			}
			if (!$series) $idseries = "NULL";
			// Procs are a special case with two possible institutions
			if ($publisher) {
				$t = new Table($this->config, "institution");
				$idinstp = $t->get_id($publisher);
			} else $idinstp = "NULL";
			if ($organization) {
				$t = new Table($this->config, "institution");
				$idinsto = $t->get_id($organization);
			} else $idinsto = "NULL";		
			$q = "insert into btw_proceedings values ('$this->bibkey', '$volume', '$url', $idseries, $idinstp, $idinsto)";				
			$c = $this->config; require("db-connect.inc.php");
			return $cnx->exec($q);			
		} else return 0;
	}		
}


class Inproceedings extends Paper { // Conference paper

	// Attributes

	public $booktitle;
	public $volume;
	public $pages;
	public $url;

	// Associations

	public $editors;
	public $organization;
	public $series;
	public $publisher;

	// Methods
	
	function __construct($config, $bibkey) { // Constructor
		// Common attributes
		parent::__construct($config, $bibkey);
		// Specific attributes
		$this->booktitle = "";
		$this->volume = "";
		$this->url = "";
		$this->pages = "";
	}	

	function load() { // Load inproceedings
		// Load common attributes
		parent::load();
		// Load editors
		$this->editors = array(); 
		$i = 0;
		$c = $this->config; require("db-connect.inc.php");
		$result = $cnx->query("select name from btw_author a, btw_editors b where a.id=b.id and bibkey='$this->bibkey' order by rank");		
		foreach ($result as $row) $this->editors[$i++] = $row["name"];	
		// Load specific attributes
		$row = parent::load_specific_attributes();
		if ($row["publisher"]) {
			$this->publisher = New Institution($this->config, $row["publisher"]);
			$this->publisher->load();
		}
		if ($row["series"]) {
			$this->series = New Series($this->config, $row["series"]);
			$this->series->load();
		}
		$this->booktitle = $row["booktitle"];
		$this->volume = $row["volume"];
		$this->pages = $row["pages"];
		if ($row["organization"]) {
			$this->organization = New Institution($this->config, $row["organization"]);
			$this->organization->load();
		}
		$this->url = $row["url"];
		// BibTeX & XML computation
		$this->bibtex .= "&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;Booktitle = {" . $this->booktitle . "},<br />\n";
		$this->xml .= "&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<booktitle>" . $this->booktitle ."</booktitle><br />\n";
		if (count($this->editors)) {
	      $this->bibtex .= "&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;Editor = {";
	      $this->xml .= "&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<editor>";
			for ($i = 0; $i < count($this->editors); $i++) {
			  if ($i > 0) {
			    $this->bibtex .= " and ";
		   	 $this->xml .= ", ";
			  }
			  $this->bibtex .= $this->editors[$i];
			  $this->xml .= $this->editors[$i];  
			}
	      $this->bibtex.="},<br />\n";
		  $this->xml.="</editor><br />\n";
		}
    	if ($this->month) {
			$this->bibtex .= "&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;Month = {" . $this->month . "},<br />\n";
			$this->xml .= "&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<month>" . $this->month . "</month><br />\n";
    	}
    	$this->bibtex .= "&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;Year = {" . $this->year . "}";
    	$this->xml .= "&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<year>" . $this->year . "</year>";
		if ($row["series"]) {
			$this->bibtex .= ",<br />\n&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;Series = {" . $this->series->name . "}";
			$this->xml .= "<br />\n&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<series>" . $this->series->name . "</series><br />\n";  
    		if ($this->volume) {
				$this->bibtex .= ",<br />\n&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;Volume = {" . $this->volume . "}";
				$this->xml .= "&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<volume>" . $this->volume . "</volume><br />\n";
			}
		}
    	if ($row["organization"]) {
    		$this->bibtex .= ",<br />\n&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;Organization = {".$this->organization->name."}";
    		$this->xml .= "&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<publisher>" . $this->organization->name . "</publisher><br />\n";
    	}	
		if ($row["publisher"]) {
    		$this->bibtex .= ",<br />\n&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;Publisher = {" . $this->publisher->name . "}";
			$this->xml .= "&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<publisher>" . $this->publisher->name . "</publisher><br />\n";
    		if ($this->publisher->address) {
    			$this->bibtex .= ",<br />\n&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;Address = {" . $this->publisher->address . "}";
    			$this->xml .= "&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<address>" . $this->publisher->address . "</address><br />\n";
			}	
		}
		if ($this->pages) {
	    	$this->bibtex .= ",<br />\n&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;Pages = {" . $this->pages . "}";
			$this->xml .= "&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<pages>" . $this->pages . "</pages><br />\n";
		}
		parent::bibtex_xml_footer();
	}

	function display($cutnpaste = "off") {
		$d = parent::display() . ", ";
	   if (($this->url) && ($cutnpaste != "on")) $d .= "<a href=\"". $this->url . "\">";
    	$d .= "<em>$this->booktitle</em>";
    	if (($this->url) && ($cutnpaste != "on")) $d .= "</a>";
		$d .= ", ";
		if (count($this->editors)) {
			for ($i = 0; $i < count($this->editors); $i++) {
				if ($i > 0) $d .= ", ";
				$namecomponents = explode(" ", $this->editors[$i]);
				$namecomponents2 = explode("-", $this->editors[$i]);
				$initial = strtoupper(substr($namecomponents[0], 0, 1)) . ".";
				$lastname = ucfirst($namecomponents[count($namecomponents)-1]);
				if (count($namecomponents) > 2) $secondinitial = strtoupper(substr($namecomponents[1], 0, 1)) . ".";
				elseif (count($namecomponents2) > 1) $secondinitial = strtoupper(substr($namecomponents2[1], 0, 1)) . ".";
				else $secondinitial = "";
				$d .= "$initial$secondinitial $lastname"; 				  
			}
			if (count($this->editors) > 1) $d .= ", Eds., ";
			else $d .= ", Ed., ";
		}
    	if ($this->month) $d .= $this->month . " ";
		$d .= $this->year;
    	if ($this->series) {
	      $d .= "; ";
	      if (($this->series->url) && ($cutnpaste != "on")) $d .= "<a href=\"" . $this->series->url . "\">";
      	$d .= "<em>" . $this->series->name . "</em>";
      	if (($this->series->url) && ($cutnpaste != "on")) $d .= "</a>";
			if ($this->volume) $d .= ", Vol. $this->volume";
			if ($this->publisher) {
				$d .= ", ";
		  		if (($this->publisher->url) && ($cutnpaste != "on")) $d .= "<a href=\"" . $this->publisher->url . "\">";
		  		$d .= $this->publisher->name;
			  	if (($this->publisher->url) && ($cutnpaste != "on")) $d .= "</a>";
				if ($this->publisher->address) $d .= ", " . $this->publisher->address;		    	
			}
		}
		if ($this->pages) $d .= ", $this->pages";
    	if (!$this->series) {		
			if ($this->publisher) {
				$d .= "; ";
		  		if (($this->publisher->url) && ($cutnpaste != "on")) $d .= "<a href=\"" . $this->publisher->url . "\">";
		  		$d .= $this->publisher->name;
			  	if (($this->publisher->url) && ($cutnpaste != "on")) $d .= "</a>";
				if ($this->publisher->address) $d .= ", " . $this->publisher->address;		    	
			}    	
			if ($this->organization) {
				$d .= "; ";
		  		if (($this->organization->url) && ($cutnpaste != "on")) $d .= "<a href=\"" . $this->organization->url . "\">";
		  		$d .= $this->organization->name;
			  	if (($this->organization->url) && ($cutnpaste != "on")) $d .= "</a>";
				if ($this->organization->address) $d .= ", " . $this->organization->address;		    	
			} 			
    	}
		return $d . parent::display_footer();
	}

	function header() {
		return $this->scope . " conferences";
	}
	
	function form($action) { // Add-modify form
		parent::form($action);
		echo "<h3>Booktitle</h3>\n<input type=\"text\" name=\"booktitle\" size=\"50\" maxlength=\"255\" value=\"$this->booktitle\" />\n";
		// Editors
		$c = $this->config; require("db-connect.inc.php");
		$q = "select name from btw_author as a, btw_editors as b where a.id=b.id and bibkey='$this->bibkey'";
		$rowcount = $cnx->query($q);		
		$n = count($rowcount->fetchAll());			
		if ($n) {
			$result = $cnx->query($q);
			echo "<h3>Editor(s)</h3>\n<ul>\n";
			foreach ($result as $row) echo "<li>" . $row["name"] . "</li>\n";	
			echo "</ul>\n";
		}
		echo "<h3>Organization</h3>\n";
		$t = new Table($this->config, "institution");
		$t->select_name("organization", $this->organization->id, 1);
		echo "<h3>Series</h3>\n";
		$t = new Table($this->config, "series");
		$t->select_name("series", $this->series->id, 1);
		echo "<h3>Volume</h3>\n<input type=\"text\" name=\"volume\" size=\"10\" maxlength=\"10\" value=\"$this->volume\" /> <span class=\"small\">optional</span>\n";
		echo "<h3>Publisher</h3>\n";
		$t = new Table($this->config, "institution");
		$t->select_name("publisher", $this->publisher->id, 1);
		echo "<h3>Pages</h3>\n<input type=\"text\" name=\"pages\" size=\"15\" maxlength=\"15\" value=\"$this->pages\" /> <span class=\"small\">optional</span>\n";
		echo "<h3>Conference web page</h3>\n<input type=\"text\" name=\"url\" size=\"50\" maxlength=\"100\" value=\"$this->url\" /> <span class=\"small\">optional</span>\n";
		parent::form_footer();
	}
	
	function save($title, $year, $month, $note, $scope, $abstract, $keywords, $tmpfile, $filename, $oldfile, $paperurl, $journalname, $volume, $number, $pages, $seriesname, $publishername, $edition, $organizationname, $booktitle, $url, $howpublished, $schoolname, $institutionname, $type) {
		if (parent::save($title, $year, $month, $note, $scope, $abstract, $keywords, $tmpfile, $filename, $oldfile, $paperurl, $journalname, $volume, $number, $pages, $seriesname, $publishername, $edition, $organizationname, $booktitle, $url, $howpublished, $schoolname, $institutionname, $type)) {	
			if ($seriesname) {
				$t = new Table($this->config, "series");
				$seriesid = $t->get_id($seriesname);
			} else $seriesid = "NULL";
			if ($publishername) {
				$t = new Table($this->config, "institution");
				$publisherid = $t->get_id($publishername);
			} else $publisherid = "NULL";
			if ($organizationname) {
				$t = new Table($this->config, "institution");
				$organizationid = $t->get_id($organizationname);
			} else $organizationid = "NULL";
			$c = $this->config; require("db-connect.inc.php");
			return $cnx->exec("insert into btw_inproceedings values ('$this->bibkey', $seriesid, $publisherid, '" . addslashes($booktitle) . "', '$volume', '$pages', $organizationid, '$url')");			
		} else return 0;
	}	
	
	function update($title, $year, $month, $note, $scope, $abstract, $keywords, $tmpfile, $filename, $oldfile, $paperurl, $journalname, $volume, $number, $pages, $seriesname, $publishername, $edition, $organizationname, $booktitle, $url, $howpublished, $schoolname, $institutionname, $type) {
		if (parent::update($title, $year, $month, $note, $scope, $abstract, $keywords, $tmpfile, $filename, $oldfile, $paperurl, $journalname, $volume, $number, $pages, $seriesname, $publishername, $edition, $organizationname, $booktitle, $url, $howpublished, $schoolname, $institutionname, $type)) {	
			if ($seriesname) {
				$t = new Table($this->config, "series");
				$seriesid = $t->get_id($seriesname);
			} else $seriesid = "NULL";
			if ($publishername) {
				$t = new Table($this->config, "institution");
				$publisherid = $t->get_id($publishername);
			} else $publisherid = "NULL";
			if ($organizationname) {
				$t = new Table($this->config, "institution");
				$organizationid = $t->get_id($organizationname);
			} else $organizationid = "NULL";
			$c = $this->config; require("db-connect.inc.php");
			$q = "update btw_inproceedings set series=$seriesid, publisher=$publisherid, booktitle='" . addslashes($booktitle) . "', volume='$volume', pages='$pages', organization=$organizationid, url='$url' where bibkey='$this->bibkey'";
			if ($cnx->exec($q) === FALSE) return 0;
			else return 1;
		} else return 0;
	}	
	
	function delete() {
		$result = parent::delete();
		$c = $this->config; require("db-connect.inc.php");
		if ($result) $result += $cnx->exec("delete from btw_editors where bibkey='$this->bibkey'");
		return $result;
	}
	
	function import($title, $year, $month, $note, $scope, $auth_id_list, $ed_id_list, $idseries, $volume, $number, $pages, $idinst, $edition, $idinstp, $booktitle, $idinsto, $howpublished, $type, $index_a, $index_e, $series, $publisher, $organization, $school, $institution, $abstract, $keywords) { // BibTeX import
		if (parent::import($title, $year, $month, $note, $scope, $auth_id_list, $ed_id_list, $idseries, $volume, $number, $pages, $idinst, $edition, $idinstp, $booktitle, $idinsto, $howpublished, $type, $index_a, $index_e, $series, $publisher, $organization, $school, $institution, $abstract, $keywords)) {
			// Editors
			$ide = explode(" ", $ed_id_list);  
			for ($i = 0; $i < $index_e; $i++) $result += mysql_query("insert into btw_editors values ('$this->bibkey', ". addslashes($ide[$i]) . ", $i+1)");	
			if (!$series) $idseries = "NULL";
			//if (!$publisher) $idinstp = "NULL";
			// Inprocs are a special case with two institutions
			if ($publisher) {
				$t = new Table($this->config, "institution");
				$idinstp = $t->get_id($publisher);
			} else $idinstp = "NULL";
			if ($organization) {
				$t = new Table($this->config, "institution");
				$idinsto = $t->get_id($organization);
			} else $idinsto = "NULL";			
			//echo "<p>\$series=$series \$idseries=$idseries \$publisher=$publisher \$organization=$organization \$idinst=$idinst \$idinstp=$idinstp \$idinsto=$idinsto</p>";
			$q = "insert into btw_inproceedings values ('$this->bibkey', $idseries, $idinstp, '" . addslashes($booktitle) . "', '$volume', '$pages', $idinsto, '')";
			//echo "<p>$q</p>\n"; // Debug
			$c = $this->config; require("db-connect.inc.php");
			return $cnx->exec($q);		
		} else return 0;
	}		
}


class Misc extends Paper { // Miscellaneous paper

	// Attributes

	public $howpublished;

	// Methods
	
	function __construct($config, $bibkey) { // Constructor
		// Common attributes
		parent::__construct($config, $bibkey);
		$this->dblpref = "www"; // DBLP's DTD doesn't know some BibTeX entries, unfortunately...
		// Specific attributes
		$this->howpublished = "";
	}	

	function load() { // Load misc
		// Load common attributes
		parent::load();
		// Load specific attribute
		$row = parent::load_specific_attributes();
		$this->howpublished = $row["howpublished"];
		// BibTeX & XML computation
    	$this->bibtex .= "&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;Howpublished = {" . $this->howpublished . "},<br />\n";
    	$this->xml .= "&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<howpublished>$this->howpublished</howpublished><br />\n";
    	if ($this->month) {
			$this->bibtex .= "&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;Month = {" . $this->month . "},<br />\n";
			$this->xml .= "&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<month>" . $this->month . "</month><br />\n";
    	}
    	$this->bibtex.="&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;Year = {" . $this->year . "}";
    	$this->xml.="&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<year>" . $this->year . "</year><br />\n";		
		parent::bibtex_xml_footer();
	}

	function display($cutnpaste = "off") {
		$d = parent::display() . ", ";
    	$d .= $this->howpublished . ", ";
    	if ($this->month) $d .= "$this->month ";
   	$d .= $this->year;
		return $d . parent::display_footer();
	}

	function header() {
		return "Other publications (" . strtolower($this->scope) . " scope)";
	}
	
	function form($action) { // Add-modify form
		parent::form($action);
		echo "<h3>Howpublished</h3>\n<input type=\"text\" name=\"howpublished\" size=\"50\" maxlength=\"255\" value=\"$this->howpublished\" />\n";
		parent::form_footer();
	}
	
	function save($title, $year, $month, $note, $scope, $abstract, $keywords, $tmpfile, $filename, $oldfile, $paperurl, $journalname, $volume, $number, $pages, $seriesname, $publishername, $edition, $organizationname, $booktitle, $url, $howpublished, $schoolname, $institutionname, $type) {
		if (parent::save($title, $year, $month, $note, $scope, $abstract, $keywords, $tmpfile, $filename, $oldfile, $paperurl, $journalname, $volume, $number, $pages, $seriesname, $publishername, $edition, $organizationname, $booktitle, $url, $howpublished, $schoolname, $institutionname, $type)) {	
			$c = $this->config; require("db-connect.inc.php");
			return $cnx->exec("insert into btw_misc values ('$this->bibkey', '" . addslashes($howpublished) . "')");				
		} else return 0;
	}	
	
	function update($title, $year, $month, $note, $scope, $abstract, $keywords, $tmpfile, $filename, $oldfile, $paperurl, $journalname, $volume, $number, $pages, $seriesname, $publishername, $edition, $organizationname, $booktitle, $url, $howpublished, $schoolname, $institutionname, $type) {
		if (parent::update($title, $year, $month, $note, $scope, $abstract, $keywords, $tmpfile, $filename, $oldfile, $paperurl, $journalname, $volume, $number, $pages, $seriesname, $publishername, $edition, $organizationname, $booktitle, $url, $howpublished, $schoolname, $institutionname, $type)) {	
			$c = $this->config; require("db-connect.inc.php");
			$q = "update btw_misc set howpublished='" . addslashes($howpublished) . "' where bibkey='$this->bibkey'";		
			if ($cnx->exec($q) === FALSE) return 0;
			else return 1;
		} else return 0;
	}	
	
	function import($title, $year, $month, $note, $scope, $auth_id_list, $ed_id_list, $idseries, $volume, $number, $pages, $idinst, $edition, $idinstp, $booktitle, $idinsto, $howpublished, $type, $index_a, $index_e, $series, $publisher, $organization, $school, $institution, $abstract, $keywords) { // BibTeX import
		if (parent::import($title, $year, $month, $note, $scope, $auth_id_list, $ed_id_list, $idseries, $volume, $number, $pages, $idinst, $edition, $idinstp, $booktitle, $idinsto, $howpublished, $type, $index_a, $index_e, $series, $publisher, $organization, $school, $institution, $abstract, $keywords)) {
			$c = $this->config; require("db-connect.inc.php");
			return $cnx->exec("insert into btw_misc values ('$this->bibkey', '" . addslashes($howpublished) . "')");			
		} else
			return 0;
	}	
}


class Phdthesis extends Paper { // PhD thesis

	// Association

	public $school;

	// Methods
	
	function __construct($config, $bibkey) { // Constructor
		// Common attributes
		parent::__construct($config, $bibkey);
		// Specific attributes
		$this->school = "";
	}	

	protected function load_specific_attributes() {
		$c = $this->config; require("db-connect.inc.php");
		$result = $cnx->query("select * from btw_phdthesis where bibkey='$this->bibkey'");			
		return $result->fetch(PDO::FETCH_ASSOC);
	}

	function load() { // Load phdthesis
		// Load common attributes
		parent::load();
		// Load specific attribute
		$row = $this->load_specific_attributes();
		$this->school = new Institution($this->config, $row["school"]);
		$this->school->load();
		// BibTeX & XML computation
    	$this->bibtex .= "&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;School = {" . $this->school->name . "},<br />\n";
    	$this->xml .= "&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<school>" . $this->school->name . "</school><br />\n";
    	if ($this->school->address) {
    		$this->bibtex .= "&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;Address = {" . $this->school->address . "},<br />\n";
    		$this->xml .= "&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<address>" . $this->school->address . "</address><br />\n";
    	}
    	if ($this->month) {
			$this->bibtex .= "&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;Month = {" . $this->month . "},<br />\n";
			$this->xml .= "&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<month>" . $this->month . "</month><br />\n";
    	}
    	$this->bibtex .= "&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;Year = {" . $this->year . "}";
    	$this->xml .= "&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<year>" . $this->year . "</year><br />\n";
		parent::bibtex_xml_footer();
	}

	function display($cutnpaste = "off") {
		$d = parent::display() . ", ";
    	if (($this->school->url) && ($cutnpaste != "on")) $d .= "<a href=\"" . $this->school->url ."\">";
    	$d .= $this->school->name;
    	if (($this->school->url) && ($cutnpaste != "on")) $d .= "</a>";
	   $d .= ", ";
    	if ($this->month) $d .= "$this->month ";
   	$d .= $this->year;
		return $d . parent::display_footer();
	}

	function header() {
		return "PhD theses";
	}
	
	function form($action) { // Add-modify form
		parent::form($action);
		echo "<h3>School</h3>\n";
		$t = new Table($this->config, "institution");
		$t->select_name("school", $this->school->id);
		parent::form_footer();
	}
	
	function save($title, $year, $month, $note, $scope, $abstract, $keywords, $tmpfile, $filename, $oldfile, $paperurl, $journalname, $volume, $number, $pages, $seriesname, $publishername, $edition, $organizationname, $booktitle, $url, $howpublished, $schoolname, $institutionname, $type) {
		if (parent::save($title, $year, $month, $note, "National", $abstract, $keywords, $tmpfile, $filename, $oldfile, $paperurl, $journalname, $volume, $number, $pages, $seriesname, $publishername, $edition, $organizationname, $booktitle, $url, $howpublished, $schoolname, $institutionname, $type)) {	
			$t = new Table($this->config, "institution");
			$schoolid = $t->get_id($schoolname);		
			$c = $this->config; require("db-connect.inc.php");
			return $cnx->exec("insert into btw_phdthesis values ('$this->bibkey', $schoolid)");				
		} else return 0;
	}	
	
	function update($title, $year, $month, $note, $scope, $abstract, $keywords, $tmpfile, $filename, $oldfile, $paperurl, $journalname, $volume, $number, $pages, $seriesname, $publishername, $edition, $organizationname, $booktitle, $url, $howpublished, $schoolname, $institutionname, $type) {
		if (parent::update($title, $year, $month, $note, "National", $abstract, $keywords, $tmpfile, $filename, $oldfile, $paperurl, $journalname, $volume, $number, $pages, $seriesname, $publishername, $edition, $organizationname, $booktitle, $url, $howpublished, $schoolname, $institutionname, $type)) {	
			$t = new Table($this->config, "institution");
			$schoolid = $t->get_id($schoolname);				
			$c = $this->config; require("db-connect.inc.php");
			$q = "update btw_phdthesis set school=$schoolid where bibkey='$this->bibkey'";
			if ($cnx->exec($q) === FALSE) return 0;
			else return 1;
		} else return 0;
	}	
	
	function import($title, $year, $month, $note, $scope, $auth_id_list, $ed_id_list, $idseries, $volume, $number, $pages, $idinst, $edition, $idinstp, $booktitle, $idinsto, $howpublished, $type, $index_a, $index_e, $series, $publisher, $organization, $school, $institution, $abstract, $keywords) { // BibTeX import
		if (parent::import($title, $year, $month, $note, "National", $auth_id_list, $ed_id_list, $idseries, $volume, $number, $pages, $idinst, $edition, $idinstp, $booktitle, $idinsto, $howpublished, $type, $index_a, $index_e, $series, $publisher, $organization, $school, $institution, $abstract, $keywords)) {
			if (!$school) $idinst = "NULL";
			$c = $this->config; require("db-connect.inc.php");
			return $cnx->exec("insert into btw_phdthesis values ('$this->bibkey', $idinst)");				  
		} else return 0;
	}		
}


class Mastersthesis extends Phdthesis { // Master's thesis

	// Methods
	
	function __construct($config, $bibkey) { // Constructor
		// Common attributes
		parent::__construct($config, $bibkey);
	}	

	function header() {
		return "Masters' theses";
	}
	
	function update_key($newkey) { 
		$c = $this->config; require("db-connect.inc.php");	
		if ($cnx->exec("update btw_paper set bibkey='$newkey', reference='$this->reference' where bibkey='$this->bibkey'") === FALSE) $result = 0;
		else $result = 1;
		if ($result) {
			$result += $cnx->exec();
			if (!($cnx->exec("update btw_phdthesis set bibkey='$newkey' where bibkey='$this->bibkey'") === FALSE)) $result += 1;
			if (!($cnx->exec("update btw_authors set bibkey='$newkey' where bibkey='$this->bibkey'") === FALSE)) $result += 1;
		}
		return $result;
    }	

	function delete() { // Cascade delete
		if ($this->file) unlink("files/" . $this->file);
		$c = $this->config; require("db-connect.inc.php");
		$result = $cnx->exec("delete from btw_paper where bibkey='$this->bibkey'");		
		if ($result) {
			$result += $cnx->exec("delete from btw_phdthesis where bibkey='$this->bibkey'");
			if ($result) {
				$result += $cnx->exec("delete from btw_authors where bibkey='$this->bibkey'");
			}
		}
		return $result;
	}	
}


class Techreport extends Paper { // Technical report

	// Attributes

	public $type;
	public $number;

	// Associations

	public $institution;

	// Methods

	function __construct($config, $bibkey) { // Constructor
		// Common attributes
		parent::__construct($config, $bibkey);
		$this->dblpref = "www"; // DBLP's DTD doesn't know some BibTeX entries, unfortunately...
		// Specific attributes
		$this->type = "";
		$this->number = "";
	}
	
	function load() { // Load techreport
		// Load common attributes
		parent::load();
		// Load specific attribute
		$row = parent::load_specific_attributes();
		$this->type = $row["type"];
		$this->number = $row["number"];
		$this->institution = new Institution($this->config, $row["institution"]);
		$this->institution->load();
		// BibTeX & XML computation
		if ($this->type) {
			$this->bibtex .= "&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;Type = {" . $this->type . "},<br />\n";
			$this->xml .= "&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<type>" . $this->type ."</type><br />\n";
    	}
    	if ($this->number) {
			$this->bibtex .= "&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;Number = {" . $this->number . "},<br />\n";
			$this->xml .= "&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<number>" . $this->number . "</number><br />\n";
    	}
    	$this->bibtex .= "&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;Institution = {" . $this->institution->name . "},<br />\n";
    	$this->xml .= "&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<publisher>" . $this->institution->name . "</publisher><br />\n";
    	if ($this->institution->address) {
    		$this->bibtex .= "&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;Address = {" . $this->institution->address . "},<br />\n";
    		$this->xml .= "&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<address>" . $this->institution->address . "</address><br />\n";
    	}    	
    	if ($this->month) {
			$this->bibtex .= "&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;Month = {" . $this->month . "},<br />\n";
			$this->xml .= "&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<month>" . $this->month ."</month><br />\n";
    	}
    	$this->bibtex .= "&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;Year = {" . $this->year . "}";
    	$this->xml .= "&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<year>" . $this->year. "</year><br />\n";	
		parent::bibtex_xml_footer();
	}

	function display($cutnpaste = "off") {
		$d = parent::display() . ", ";
	   if ($this->type) $d .= "$this->type, ";
    	if ($this->number) $d .= "No. $this->number, ";
    	if (($this->institution->url) && ($cutnpaste != "on")) $d .= "<a href=\"" . $this->institution->url ."\">";
    	$d .= $this->institution->name;
	   if (($this->institution->url) && ($cutnpaste != "on")) $d .= "</a>";
    	$d .= ", ";
    	if ($this->month) $d .= "$this->month ";
   	$d .= $this->year;
		return $d . parent::display_footer();
	}

	function header() {
		return "Technical reports";
	}
	
	function form($action) { // Add-modify form
		parent::form($action);
	    echo "<h3>Institution</h3>\n";
		$t = new Table($this->config, "institution");
		$t->select_name("institution", $this->institution->id);
		echo "<h3>Type</h3>\n<input type=\"text\" name=\"type\" size=\"50\" maxlength=\"50\" value=\"$this->type\" /> <span class=\"small\">optional</span>\n";
		echo "<h3>Number</h3>\n<input type=\"text\" name=\"number\" size=\"15\" maxlength=\"15\" value=\"$this->number\" /> <span class=\"small\">optional</span>\n";
		parent::form_footer();
	}
	
	function save($title, $year, $month, $note, $scope, $abstract, $keywords, $tmpfile, $filename, $oldfile, $paperurl, $journalname, $volume, $number, $pages, $seriesname, $publishername, $edition, $organizationname, $booktitle, $url, $howpublished, $schoolname, $institutionname, $type) {
		if (parent::save($title, $year, $month, $note, "National", $abstract, $keywords, $tmpfile, $filename, $oldfile, $paperurl, $journalname, $volume, $number, $pages, $seriesname, $publishername, $edition, $organizationname, $booktitle, $url, $howpublished, $schoolname, $institutionname, $type)) {	
			$t = new Table($this->config, "institution");
			$institutionid = $t->get_id($institutionname);		
			$c = $this->config; require("db-connect.inc.php");
			return $cnx->exec("insert into btw_techreport values ('$this->bibkey', $institutionid, '" . addslashes($type) ."', '$number')");			
		} else return 0;
	}	
	
	function update($title, $year, $month, $note, $scope, $abstract, $keywords, $tmpfile, $filename, $oldfile, $paperurl, $journalname, $volume, $number, $pages, $seriesname, $publishername, $edition, $organizationname, $booktitle, $url, $howpublished, $schoolname, $institutionname, $type) {
		if (parent::update($title, $year, $month, $note, "National", $abstract, $keywords, $tmpfile, $filename, $oldfile, $paperurl, $journalname, $volume, $number, $pages, $seriesname, $publishername, $edition, $organizationname, $booktitle, $url, $howpublished, $schoolname, $institutionname, $type)) {	
			$t = new Table($this->config, "institution");
			$institutionid = $t->get_id($institutionname);		
			$c = $this->config; require("db-connect.inc.php");
			$q = "update btw_techreport set institution=$institutionid, type='" . addslashes($type) ."', number='$number' where bibkey='$this->bibkey'";
			if ($cnx->exec($q) === FALSE) return 0;
			else return 1;
		} else return 0;
	}	
	
	function import($title, $year, $month, $note, $scope, $auth_id_list, $ed_id_list, $idseries, $volume, $number, $pages, $idinst, $edition, $idinstp, $booktitle, $idinsto, $howpublished, $type, $index_a, $index_e, $series, $publisher, $organization, $school, $institution, $abstract, $keywords) { // BibTeX import
		if (parent::import($title, $year, $month, $note, "National", $auth_id_list, $ed_id_list, $idseries, $volume, $number, $pages, $idinst, $edition, $idinstp, $booktitle, $idinsto, $howpublished, $type, $index_a, $index_e, $series, $publisher, $organization, $school, $institution, $abstract, $keywords)) {
	        if (!$institution) $idinst = "NULL";
			$c = $this->config; require("db-connect.inc.php");
			return $cnx->exec("insert into btw_techreport values ('$this->bibkey', $idinst, '". addslashes($type) . "', '$number')");
		} else return 0;
	}		
}


// BTW interface


class Display extends Cfg {

	function __construct($config) { // Constructor 
		parent::__construct($config);
	}

	// Private methods

	private function inline_options($publi) { // Display reference inline options (e.g., BibTeX)
		$d = "[ <a href=\"" . $_SERVER["PHP_SELF"] ."?action=bibtex&amp;bibkey=" . $publi->bibkey . "&amp;refr=" . $publi->reference . "\" class=\"utilink\">BibTeX</a>";
		$d .= " | <a href=\"" . $_SERVER["PHP_SELF"] ."?action=xml&amp;bibkey=" . $publi->bibkey . "&amp;refr=" . $publi->reference . "\" class=\"utilink\">XML</a>";
		if ($publi->abstract) $d .= " | <a href=\"" . $_SERVER["PHP_SELF"] ."?action=abstract&amp;bibkey=" . $publi->bibkey . "&amp;refr=" . $publi->reference . "\" class=\"utilink\">Abstract</a>";
		if (($publi->file) || ($publi->paperurl)) $d .= " | <a href=\"" . $_SERVER["PHP_SELF"] ."?action=fullpaper&amp;bibkey=" . $publi->bibkey . "&amp;refr=" . $publi->reference . "\" class=\"utilink\">Full paper</a>";
		return $d;
	}

	private function ref_header($publi) { // Display reference header
		echo "<h3>Reference ($publi->reference)</h3>\n\n";
		echo "<p>" . $publi->display("on") . "</p>\n\n";
	}

	private function search_authed(&$list_glob, $table, $name="") { // Search author or editor by name/keyword
		if (!$name) $name = $_SESSION["keyword"];
    	// Search keywords in author or editor list
		$c = $this->config; require("db-connect.inc.php");
		$result = $cnx->query("select distinct btw_paper.bibkey from btw_paper, btw_$table, btw_author where btw_paper.bibkey = btw_$table.bibkey and btw_$table.id = btw_author.id and upper(name) like upper('%".$name."%')");		
    	$list_test = "";	
    	$i = 0;
		foreach ($result as $row) {
	    	if ($i) $list_test .= ", ";
	    	$list_test .= "'" . $row["bibkey"] . "'";
	    	$i++;
	    }
		if ($list_glob) if ($list_test) $list_glob .= ", ";
		$list_glob .= $list_test;
		//echo "list_glob=$list_glob<br />"; // Debug
   }
   	
	private function search_authors_by_keyword() { // Pretty explicit method name, uh? Returns an IN SQL clause for inclusion in another SQL query
		$list = "";
		$this->search_authed($list, "authors");
		$this->search_authed($list, "editors");
		$this->search_authed($list, "book_editors");
		$this->search_authed($list, "proc_editors");
  	 	if ($list) $list = " or bibkey in ($list) ";
  	 	return $list;
	}
	
	private function mgt_form_header($action) { // Management form header
		echo "<form action=\"" . $_SERVER["PHP_SELF"] . "\" method=\"post\" enctype=\"multipart/form-data\"><fieldset>\n";
		echo "<input type=\"hidden\" name=\"action\" value=\"$action\" />\n";	
	}
	
	private function mgt_form_footer($label, $name = "submit") { // Management form footer
		echo "<input type=\"submit\" name=\"$name\" value=\"$label\" />\n";	
		echo "</fieldset></form>\n\n";
	}
	
	private function select_ref($action) { // Reference selection list
		$this->mgt_form_header("form_" . strtolower($action) . "_ref");
		echo "<select name=\"bibkey\">\n";
		$c = $this->config; require("db-connect.inc.php");
		$result = $cnx->query("select bibkey from btw_paper order by bibkey");
		foreach ($result as $row) echo "<option>" . $row["bibkey"] . "</option>\n";
		echo "</select>\n";
		$this->mgt_form_footer($action);
	}	
	
	private function add_button_asi($table) { // Add button for authors, series, and institutions
		$this->mgt_form_header("form_add_asi");
		echo "<input type=\"hidden\" name=\"table\" value=\"$table\" />\n";
		$this->mgt_form_footer("Add " . $table, "add");
	}	
	
	private function select_asi($table, $action) { // Select list for authors, series, and institutions
		$this->mgt_form_header("form_" . strtolower($action) . "_asi");	
		echo "<input type=\"hidden\" name=\"table\" value=\"$table\" />\n";		
		$t = new Table($this->config, $table);
		$t->select_name();
		$this->mgt_form_footer($action);
	}	
   	 
	private function return_button() { // Return button (to management menu)
		echo "<form action=\"" . $_SERVER["PHP_SELF"] . "\" method=\"post\" enctype=\"multipart/form-data\"><fieldset>\n";
		echo "<input type=\"submit\" name=\"Back\" value=\"Back to menu\" />\n";
		echo "</fieldset></form>\n\n";
	}
	 
	private function search_author($searchtype, $id, $step, $req, $reference, $scope, $bibkey, $author, $editor, $title, $year, $month, $note, $journal, $volume, $number, $pages, $series, $publisher, $edition, $booktitle, $organization, $school, $howpublished, $institution, $type, $address, $auth_id_list, $index_a, $ed_id_list, $index_e, $idseries, $idinst, $abstract, $keywords) { // Author/editor DB search (BibTeX import)
		if ($searchtype == "author") {
			$line = explode(" and ", $author); 
			$names = explode(" ", $line[$index_a]);
		} else {
			$line = explode(" and ", $editor); 
			$names = explode(" ", $line[$index_e]);
		}
		$uppn = addslashes(strtoupper($names[count($names)-1]));
		// Prepare for next auth/edit
		if ($searchtype == "author") {
			$index = $index_a;
			$index_a++;
		} else {
			$index = $index_e;
			$index_e++;
		}
		// Look for similar last names in the database
		$q = "select * from btw_author where upper(name) like '%$uppn%'";
		//echo "<p>$q</p>\n"; // Debug
		$c = $this->config; require("db-connect.inc.php");
		$rowcount = $cnx->query($q);
		$n = count($rowcount->fetchAll());
		$result = $cnx->query($q);
		foreach ($result as $row) {		
			$idauth = $row["id"];
			$name = $row["name"];
			echo "<p>" . ucfirst($searchtype) . " <strong>$line[$index]</strong> looks like $searchtype <strong>$name</strong> from the database.</p>\n";
			$this->next_step($id, $step, $req, $reference, $scope, $bibkey, $author, $editor, $title, $year, $month, $note, $journal, $volume, $number, $pages, $series, $publisher, $edition, $booktitle, $organization, $school, $howpublished, $institution, $type, $address, $auth_id_list, $index_a, $ed_id_list, $index_e, $idseries, $idinst, $abstract, $keywords, "Select ".$searchtype, $idauth);
		}
		if ($n) echo "<p>OR</p>\n\n";
		echo "<p>Insert $searchtype <strong>$line[$index]</strong> in the database.</p>\n\n";
		$this->next_step($id, $step, $req, $reference, $scope, $bibkey, $author, $editor, $title, $year, $month, $note, $journal, $volume, $number, $pages, $series, $publisher, $edition, $booktitle, $organization, $school, $howpublished, $institution, $type, $address, $auth_id_list, $index_a, $ed_id_list, $index_e, $idseries, $idinst, $abstract, $keywords, "Insert ".$searchtype, $line[$index]);
	}

	private function search_series($searchtype, $id, $step, $req, $reference, $scope, $bibkey, $author, $editor, $title, $year, $month, $note, $journal, $volume, $number, $pages, $series, $publisher, $edition, $booktitle, $organization, $school, $howpublished, $institution, $type, $address, $auth_id_list, $index_a, $ed_id_list, $index_e, $idseries, $idinst, $abstract, $keywords) { // Journal/series DB search (BibTeX import)
		if ($searchtype == "journal") $iname = $journal;
		else $iname = $series;
		$uppn = addslashes(strtoupper($iname));
		// Look for similar names in the database
		$c = $this->config; require("db-connect.inc.php");
		$q = "select * from btw_series where upper(name) like '%$uppn%'";
		$rowcount = $cnx->query($q);
		$n = count($rowcount->fetchAll());
		$result = $cnx->query($q);
		foreach ($result as $row) {	
			$idseries = $row["id"];
			$name = $row["name"];
			$url = $row["url"];
			echo "<p>" . ucfirst($searchtype) . " <strong>$iname</strong> looks like $searchtype <strong>$name</strong> ";
			if ($url) echo "(<a href=\"https://$url\">$url</a>)";
			echo " from the database.</p>\n\n";
			$this->next_step($id, $step, $req, $reference, $scope, $bibkey, $author, $editor, $title, $year, $month, $note, $journal, $volume, $number, $pages, $series, $publisher, $edition, $booktitle, $organization, $school, $howpublished, $institution, $type, $address, $auth_id_list, $index_a, $ed_id_list, $index_e, $idseries, $idinst, $abstract, $keywords, "Select ".$searchtype);
		}
		if ($n) echo "<p>OR</p>\n\n";
		echo "<p>Insert $searchtype <strong>$iname</strong> in the database.</p>\n\n";
		$this->next_step($id, $step, $req, $reference, $scope, $bibkey, $author, $editor, $title, $year, $month, $note, $journal, $volume, $number, $pages, $series, $publisher, $edition, $booktitle, $organization, $school, $howpublished, $institution, $type, $address, $auth_id_list, $index_a, $ed_id_list, $index_e, $idseries, $idinst, $abstract, $keywords, "Insert ".$searchtype, $iname);
	}

	private function search_inst($searchtype, $id, $step, $req, $reference, $scope, $bibkey, $author, $editor, $title, $year, $month, $note, $journal, $volume, $number, $pages, $series, $publisher, $edition, $booktitle, $organization, $school, $howpublished, $institution, $type, $address, $auth_id_list, $index_a, $ed_id_list, $index_e, $idseries, $idinst, $abstract, $keywords) { // Institution/school/publisher/organization DB search (BibTeX import)
		switch ($searchtype) {
			case "publisher": $iname = $publisher; break;
			case "institution": $iname = $institution; break;
			case "organization": $iname = $organization; break;
			case "school": $iname = $school; break;
		}
		$uppn = addslashes(strtoupper($iname));
		// Look for similar names in the database
		$c = $this->config; require("db-connect.inc.php");
		$q = "select * from btw_institution where upper(name) like '%$uppn%'";
		$rowcount = $cnx->query($q);
		$n = count($rowcount->fetchAll());
		$result = $cnx->query($q);
		foreach ($result as $row) {	
			$idinst = $row["id"];
			$name =  $row["name"];
			$iaddress =  $row["address"];
			$url =  $row["url"];		
			echo "<p>" . ucfirst($searchtype) . " <strong>$iname</strong>";
			if ($address) echo ", $address";		
			echo " looks like $searchtype <strong>$name</strong>";
			if ($iaddress) echo ", $iaddress";
			if ($url) echo " (<a href=\"https://$url\">$url</a>)";
			echo " from the database.</p>\n\n";
			$this->next_step($id, $step, $req, $reference, $scope, $bibkey, $author, $editor, $title, $year, $month, $note, $journal, $volume, $number, $pages, $series, $publisher, $edition, $booktitle, $organization, $school, $howpublished, $institution, $type, $address, $auth_id_list, $index_a, $ed_id_list, $index_e, $idseries, $idinst, $abstract, $keywords, "Select ".$searchtype);
		}
		if ($n) echo "<p>OR</p>\n\n";
		echo "<p>Insert $searchtype <strong>$iname</strong> in the database.</p>\n\n";
		$this->next_step($id, $step, $req, $reference, $scope, $bibkey, $author, $editor, $title, $year, $month, $note, $journal, $volume, $number, $pages, $series, $publisher, $edition, $booktitle, $organization, $school, $howpublished, $institution, $type, $address, $auth_id_list, $index_a, $ed_id_list, $index_e, $idseries, $idinst, $abstract, $keywords, "Insert ".$searchtype, $iname);
	}		
	 
	// Public methods

	function msg($result) { // Generic message after database operation
		if ($result) echo "<p>Update done.</p>\n\n";
		else echo "<p><strong>Update error!</strong></p>\n\n";
		$this->return_button();
	}
	
	function new_publi($reference, $bibkey) { // Create new publication wrt type
		// echo "reference=$reference bibkey=$bibkey<br />"; // Debug
		switch ($reference) {
        	case "article": 			$publi = new Article($this->config, $bibkey); break;
        	case "book": 				$publi = new Book($this->config, $bibkey); break;
        	case "inbook": 			    $publi = new Inbook($this->config, $bibkey); break;
        	case "inproceedings": 	    $publi = new Inproceedings($this->config, $bibkey); break;
        	case "mastersthesis":	    $publi = new Mastersthesis($this->config, $bibkey); break;
        	case "misc":				$publi = new Misc($this->config, $bibkey); break;
        	case "phdthesis": 		    $publi = new Phdthesis($this->config, $bibkey); break;
        	case "techreport":		    $publi = new Techreport($this->config, $bibkey); break;
        	case "proceedings": 		$publi = new Proceedings($this->config, $bibkey); break;
		}
		return $publi;
	}
	
	function list_all($homepage, $username) { // List all publications in the database
		// Init
		$currentreftype = "";
    	$currentscope = "";
		$_SESSION["bigbibtex"] = "";
		$_SESSION["bigxml"] = "";
    	// Year selection
    	echo "<form action=\"" . $_SERVER["PHP_SELF"] . "\" method=\"post\" enctype=\"multipart/form-data\"><fieldset>\n";
    	echo "<span class=\"small\">From:</span> <input type=\"text\" name=\"minyear\" size=\"4\" maxlength=\"4\" value=\"" . $_SESSION["minyear"] . "\" />\n";
	 	echo "<span class=\"small\">To:</span> <input type=\"text\" name=\"maxyear\" size=\"4\" maxlength=\"4\" value=\"" . $_SESSION["maxyear"] . "\" />\n";
	 	echo "<span class=\"small\">Keyword filter:</span> <input type=\"text\" name=\"keyword\" size=\"15\" maxlength=\"50\" value=\"" . $_SESSION["keyword"] . "\" />\n";
	 	echo "<span class=\"small\">Cut &amp; paste mode:</span> <input type=\"checkbox\" name=\"cutnpaste\""; 
		if ($_SESSION["cutnpaste"] == "on") echo " checked=\"checked\"";
		echo " />\n";
    	echo "<input type=\"submit\" value=\"OK\" />\n";
    	echo "</fieldset></form>\n\n";
    	// Display list
		$q = "select bibkey, reference from btw_paper, btw_reference where reference=type and yea between " . $_SESSION["minyear"] . " and " . $_SESSION["maxyear"] . " and (upper(title) like upper('%" . $_SESSION["keyword"] . "%') or upper(keywords) like upper('%" . $_SESSION["keyword"] . "%') " . $this->search_authors_by_keyword() . ") order by scope, rank, yea desc";
		// echo "<p>$q</p>\n"; // Debug
		$c = $this->config; require("db-connect.inc.php");
		$result = $cnx->query($q);
		foreach ($result as $row) {			
			// Create publication
			$publi = $this->new_publi($row["reference"], $row["bibkey"]);
			$publi->load();
			// BibTeX & XML management
			$_SESSION["bigbibtex"] .= $publi->bibtex;
			$_SESSION["bigxml"] .= $publi->xml;
			// Display publication header if necessary
			if (($publi->reference != $currentreftype) || ($publi->scope != $currentscope)) {
				echo "<h3>" . $publi->header() . "</h3>\n\n";
        		$currentreftype = $publi->reference;
        		$currentscope = $publi->scope;
			}
			// Display reference proper
			echo "<p id=\"$publi->bibkey\">" . $publi->display($_SESSION["cutnpaste"]);
			// Inline options
			if ($_SESSION["cutnpaste"] != "on") echo " <span class=\"small\">" . $this->inline_options($publi) . " ]</span>";
			echo "</p>\n\n";
    	}
	   // General options	
		echo "<p id=\"footoptions\">&nbsp;</p>\n";
		echo "<p>[ <a href=\"" . $_SERVER["PHP_SELF"] . "?action=bibtexall\" class=\"utilink\">BibTeX</a>";
		echo " | <a href=\"" . $_SERVER["PHP_SELF"] . "?action=xmlall\" class=\"utilink\">XML</a>";
		echo " | <a href=\"" . $_SERVER["PHP_SELF"] . "?action=stats\" class=\"utilink\">Statistics</a>";
		echo " | <a href=\"" . $_SERVER["PHP_SELF"] . "?action=index\" class=\"utilink\">Author index</a>";		
		echo " | <a href=\"#\" class=\"utilink\">Back to top</a>";
		if ($homepage) echo " | <a href=\"$homepage\" class=\"utilink\">Back to $username's homepage</a>";
		echo " ]</p>\n\n";
	}

	function asbtract($reference, $bibkey) { // Display abstract
		$publi = $this->new_publi($reference, $bibkey);
		$publi->load();
		$this->ref_header($publi);
      echo "<h3>Abstract</h3>\n\n<p>";
      echo nl2br($publi->abstract);
      echo "</p>\n\n<h3>Keywords</h3>\n\n<p>";
      echo $publi->keywords;
      echo "</p>\n\n<p>&nbsp;</p>\n\n";
		$publi->abstract = FALSE;
		echo "<p>" . $this->inline_options($publi) . " | <a href=\"" . $_SERVER["PHP_SELF"] . "#" . $publi->bibkey . "\" class=\"utilink\">Back</a> ]</p>\n\n";
	}
	
	function bibtex($reference, $bibkey) { // Display BibTeX
		$publi = $this->new_publi($reference, $bibkey);
		$publi->load();  
		$this->ref_header($publi);
		echo "<h3>BibTeX entry</h3>\n\n<p>$publi->bibtex</p>\n\n";
		echo "<p>[ <a href=\"export.php?bibkey=" . $publi->bibkey . "&amp;refr=" . $publi->reference . "\" class=\"utilink\">Export</a> | <a href=\"" . $_SERVER["PHP_SELF"] . "#" . $publi->bibkey . "\" class=\"utilink\">Back</a> ]</p>\n\n";
	}

	function bibtex_export($reference, $bibkey) { // Export BibTeX	(one reference or all, same code)
		if ($reference) {
			$publi = $this->new_publi($reference, $bibkey);
			$publi->load();
			$output = $publi->bibtex;  
		} else $output = str_replace("\\\'", "'", $_SESSION["bigbibtex"]);
		$output = str_replace("\'", "'", $output);
        $output = str_replace("&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;", "\t", $output);
        $output = str_replace("<br />", "", $output);
		return "\n% BibTeX file $bibkey.bib generated by BibTeXWeb on " . date("F j, Y") . ".\n\n" . $output;
	}
	
	function xml($username, $reference = "", $bibkey = "") { // Export XML (one reference or all, same code)
		if ($reference) {
			$publi = $this->new_publi($reference, $bibkey);
			$publi->load();
			$this->ref_header($publi);	
			$xml = $publi->xml;
			$ext = "(excerpt)";
			$back = $_SERVER["PHP_SELF"] . "#" . $publi->bibkey;
		} else {
			$xml = $_SESSION["bigxml"];
			$ext = "";
			$back = $_SERVER["PHP_SELF"] . "#footoptions";
		}
		echo "<h3>XML DBLP entry</h3>\n\n";
		$xmlheader = "<?xml version=\"1.0\" encoding=\"iso-8859-1\" ?>\n\n<!-- Publications of $username $ext -->\n<!-- XML file btw.xml generated by BibTeXWeb on " . date("F j, Y") . " -->\n\n<!DOCTYPE dblp SYSTEM \"https://dblp.uni-trier.de/xml/dblp.dtd\">\n\n";
		$xmlheader = "<?xml version=\"1.0\" encoding=\"utf-8\" ?>\n\n<!-- Publications of $username $ext -->\n<!-- XML file btw.xml generated by BibTeXWeb on " . date("F j, Y") . " -->\n\n<!DOCTYPE dblp SYSTEM \"https://dblp.uni-trier.de/xml/dblp.dtd\">\n\n";        
		$xmlbody = "<dblp><br />\n" . str_replace('#', '"', $xml) . "</dblp>\n";
        $xml = str_replace("\'", "'", $xmlheader . $xmlbody);
        $xml = str_replace("&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;", "\t", $xml);
        $xml = str_replace("<br />", "\n", $xml);
        $xmldisplay = str_replace(">", "&gt;", str_replace("<", "&lt;", $xmlbody));
        $xmldisplay = str_replace("&lt;br /&gt;", "<br />", $xmldisplay);
        echo "<p>$xmldisplay</p>\n\n";
        $handle = fopen("./files/btw.xml", "w");
        fwrite($handle, $xml);
        fclose($handle);
        echo "<p>[ <a href=\"./files/btw.xml\" class=\"utilink\">Export</a> | <a href=\"$back\" class=\"utilink\">Back</a> ]</p>\n";	
	}
	
	function fullpaper($reference, $bibkey) { // Download full paper
		$publi = $this->new_publi($reference, $bibkey);
		$publi->load();
		$this->ref_header($publi);
		echo "<h3>Full paper</h3>\n\n";
		if ($publi->file) {
			$path = "files/" . $publi->file;
			$size = floor(filesize($path)/1024);
			echo "<pre><a href=\"$path\">". $publi->file . "</a> ($size KB)</pre>\n\n";
		} else echo "<p><a href=\"" . $publi->paperurl . "\">" . $publi->paperurl . "</a></p>\n\n"; // URL
		echo "<p>[ <a href=\"" . $_SERVER["PHP_SELF"] . "#" . $publi->bibkey . "\" class=\"utilink\">Back</a> ]</p>\n\n";
	}
	
	function bibtexall($username) { // Display full BibTeX
	  echo "<h3>BibTeX entries</h3>\n\n<p>" . str_replace("\'", "'", $_SESSION["bigbibtex"]) . "</p>\n\n";
		$namecomponents = explode(" ", $username);
     	$lastname = strtolower($namecomponents[count($namecomponents)-1]);      
		echo "<p>[ <a href=\"export.php?bibkey=$lastname\" class=\"utilink\">Export</a> | <a href=\"" . $_SERVER["PHP_SELF"] . "#footoptions\" class=\"utilink\">Back</a> ]</p>\n\n";
	}	
	
	function stats() { // Display database stats
		echo "<h3>Statistics from " . $_SESSION["minyear"] . " to " . $_SESSION["maxyear"];
		if ($_SESSION["keyword"]) echo " with keyword = \"" . $_SESSION["keyword"] ."\"";
		echo "</h3>\n\n";
      echo "<table>\n<tr> <th>Reference type</th> <th>Scope</th> <th>Number of references</th> </tr>\n";
      $tot = 0;	  
	  $c = $this->config; require("db-connect.inc.php");
	  $result = $cnx->query("select reference, scope, count(*) as c from btw_paper, btw_reference where reference=type and yea between " . $_SESSION["minyear"] . " and " . $_SESSION["maxyear"] . " and (upper(title) like upper('%" . $_SESSION["keyword"] ."%') or upper(keywords) like upper('%" . $_SESSION["keyword"] ."%') " . $this->search_authors_by_keyword() . ") group by reference, scope order by scope, rank");
	  foreach ($result as $row) {			
        $tot += $row["c"];
        echo "<tr> <td>" . $row["reference"] . "</td> <td>" . $row["scope"] . "</td> <td class=\"right\">" . $row["c"] . "</td> </tr>\n";
      }
      echo "<tr> <td><strong>Total</strong></td> <td></td> <td class=\"right\"><strong>$tot</strong></td> </tr>\n";
      echo "</table>\n\n";
		echo "<p>[ <a href=\"" . $_SERVER["PHP_SELF"] . "#footoptions\" class=\"utilink\">Back</a> ]</p>\n\n";    
	}
	
	function author_index() { // Display list of authors
		echo "<h3>Author index</h3>\n\n";
		echo "<ol>\n";		
	    $c = $this->config; require("db-connect.inc.php");
	    $result = $cnx->query("select distinct name from btw_author order by name");
	    foreach ($result as $row) echo "<li> " . $row["name"] . "</li>\n";	
		echo "</ol>\n\n";
		echo "<p>[ <a href=\"" . $_SERVER["PHP_SELF"] . "#footoptions\" class=\"utilink\">Back</a> ]</p>\n\n";
	}
	
	function install_phase1($action = "create") { // Install config file interface
		if ($action == "update") require("config.inc.php");
		if (!isset($hostname)) $hostname = "localhost";			
		if (!isset($cssname)) $cssname = "default.css";			
		if (!isset($username)) $username = "";						
		if (!isset($homepage)) $homepage = "";						
		if (!isset($login)) $login = "";							
		if (!isset($passwd)) $passwd = "";							
		if (!isset($dbname)) $dbname = "";									
		if (!isset($rsslength)) $rsslength = "10";
		if (!isset($emailalert)) $emailalert = "";		
		if (!isset($timezone)) $timezone = "Europe/Paris";
		echo "<form action=\"" . $_SERVER["PHP_SELF"] . "\" method=\"post\" enctype=\"multipart/form-data\">\n";
		echo "<fieldset>\n";
		echo "<input type=\"hidden\" name=\"action\" value=\"create_config\" />\n";
		if ($action == "create") echo "<input type=\"hidden\" name=\"db_create\" value=\"1\" />\n";
		echo "<h3>User (full) name</h3>\n<p><input type=\"text\" name=\"username\" size=\"20\" value=\"$username\" /></p>\n";
		echo "<h3>User homepage (optional)</h3>\n<p><input type=\"text\" name=\"homepage\" size=\"20\" value=\"$homepage\" /></p>\n";
		echo "<h3>MySQL hostname</h3>\n<p><input type=\"text\" name=\"hostname\" size=\"20\"  value=\"$hostname\" /></p>\n";
		echo "<h3>MySQL login</h3>\n<p><input type=\"text\" name=\"login\" size=\"20\" value=\"$login\" /></p>\n";
		echo "<h3>MySQL password</h3>\n<p><input type=\"password\" name=\"passwd\" size=\"20\" value=\"$passwd\" /></p>\n";
		echo "<h3>MySQL database name</h3>\n<p><input type=\"text\" name=\"dbname\" size=\"20\" value=\"$dbname\" /></p>\n";
		echo "<h3>Stylesheet name</h3>\n<p><input type=\"text\" name=\"cssname\" size=\"20\" value=\"$cssname\" /></p>\n";
		echo "<h3>RSS feed maximum length</h3>\n<p><input type=\"text\" name=\"rsslength\" size=\"3\" value=\"$rsslength\" /></p>\n";
		echo "<h3>Alert e-mail address (optional)</h3>\n<p><input type=\"text\" name=\"emailalert\" size=\"20\" value=\"$emailalert\" /></p>\n";	
		echo "<h3>Time zone</h3>\n<p><input type=\"text\" name=\"timezone\" size=\"20\" value=\"$timezone\" /></p>\n";
		echo "<p><input type=\"reset\" name=\"Reset\" value=\"Reset\" />\n";
		echo "<input type=\"submit\" name=\"Submit\" value=\"Submit\" /></p>\n";
		echo "</fieldset>\n";
		echo "</form>\n\n";
	}	
	
	function install_phase2() { // Install casual interface
		echo "<form action=\"" . $_SERVER["PHP_SELF"] . "\" method=\"post\"><fieldset><input type=\"hidden\" name=\"action\" value=\"modify_config\"><input type=\"submit\" value=\"Modify configuration\"></fieldset></form>\n\n";
		echo "<form action=\"" . $_SERVER["PHP_SELF"] . "\" method=\"post\"><fieldset><input type=\"hidden\" name=\"action\" value=\"destroy_db_confirm\"><input type=\"submit\" value=\"Destroy database\"></fieldset></form>\n\n";
		echo "<form action=\"" . $_SERVER["PHP_SELF"] . "\" method=\"post\"><fieldset><input type=\"hidden\" name=\"action\" value=\"create_db\"><input type=\"submit\" value=\"Re-create database\"></fieldset></form>\n\n";
		echo "<form action=\"" . $_SERVER["PHP_SELF"] . "\" method=\"post\"><fieldset><input type=\"hidden\" name=\"action\" value=\"logout\"><input type=\"submit\" value=\"Log out\"></fieldset></form>\n\n";
	}
	
	function dbdestroy_form() { // Database destruction confirmation form
      echo "<p>Confirm database destruction:</p>\n";
      echo "<div class=\"page\"><div class=\"column\"><form action=\"" . $_SERVER["PHP_SELF"] . "\" method=\"post\" enctype=\"multipart/form-data\"><fieldset>\n";
      echo "<input type=\"submit\" name=\"Cancel\" value=\"Cancel\" /></fieldset></form></div>\n";
      echo "<div class=\"column\"><form action=\"" . $_SERVER["PHP_SELF"] . "\" method=\"post\" enctype=\"multipart/form-data\"><fieldset>\n";
      echo "<input type=\"hidden\" name=\"action\" value=\"destroy_db\" />\n";
      echo "<input type=\"submit\" name=\"Destroy\" value=\"Destroy\" /></fieldset></form></div></div>\n\n";	
	}	
	
	function identification() { // Ask user to identify with MySQL login ($clogin) and password ($cpasswd)
		echo "<form action=\"" . $_SERVER["PHP_SELF"] . "\" method=\"post\" enctype=\"multipart/form-data\"><fieldset>\n";
		echo "<h3>MySQL login</h3><p><input type=\"text\" name=\"clogin\" size=\"20\" /></p>\n";
		echo "<h3>MySQL password</h3><p><input type=\"password\" name=\"cpasswd\" size=\"20\" /></p>\n";
		echo "<p><input type=\"reset\" name=\"Reset\" value=\"Reset\" /> \n";
		echo "<input type=\"submit\" name=\"Submit\" value=\"Submit\" /></p>\n";
		echo "</fieldset></form>\n\n";
	}	
	
	function mgt_menu() { // Display management main forms (insert/delete/update references/authors/journals/institutions...)
		// Logout
		$this->mgt_form_header("logout");
		$this->mgt_form_footer("Log out");
		// Reference submenu
		$t = new Table($this->config, "paper");
		echo "<h2>Reference (" . $t->count() . ")</h2>\n\n";
    	echo "<p>Before adding a reference, be sure to add the author(s), editor(s), journal, series, publisher, school, institution, and/or organization into the database <strong>first</strong>.</p>\n\n";
		$this->mgt_form_header("add_ref_key");    	
		$this->mgt_form_footer("Add reference", "add");
		$this->select_ref("Modify");
		$this->select_ref("Delete");
		// Author/editor submenu
		$t = new Table($this->config, "author");
		echo "<h2>Author or Editor (" . $t->count() . ")</h2>\n\n";
    	$this->add_button_asi("author");
    	$this->select_asi("author", "Modify");
    	$this->select_asi("author", "Delete");	
		// Journal/series submenu
		$t = new Table($this->config, "series");
		echo "<h2>Journal or Series (" . $t->count() . ")</h2>\n\n";
		$this->add_button_asi("series");
		$this->select_asi("series", "Modify");
		$this->select_asi("series", "Delete");
		// Publisher/school/institution/org submenu
		$t = new Table($this->config, "institution");
		echo "<h2>Publisher, School, Institution, or Organization (" . $t->count() . ")</h2>\n\n";
		$this->add_button_asi("institution");
		$this->select_asi("institution", "Modify");
		$this->select_asi("institution", "Delete");		
		// BibTeX import submenu
		echo "<h2>BibTeX import</h2>\n\n";
		echo "<form action=\"" . $_SERVER["PHP_SELF"] . "\" method=\"post\" enctype=\"multipart/form-data\"><fieldset>\n";
		echo "<input type=\"hidden\" name=\"action\" value=\"import_load\" />\n";
		echo "<p>BibTeX file: <input type=\"file\" name=\"file\" size=\"30\" maxlength=\"255\" /></p>\n";
		echo "<p><input type=\"submit\" value=\"Import\" /></p>\n";
		echo "</fieldset></form>\n\n";
	}
	
	function logout() { // Exit management/install interface
		 unset($_SESSION["login"]); // session_destroy() needs to be executed twice, so...
		 echo "<p>You are disconnected from BTW's management or install interface. Go to <a href=\"./\">bibliography display</a>.</p>\n\n";
	}
	
	function new_asi($table, $id) { // Create new author|series|institution
		switch($table) {
			case "author":		$asi = new Author($this->config, $id); break;
			case "series": 		$asi = new Series($this->config, $id); break;
			case "institution":	$asi = new Institution($this->config, $id); break;
		}
		return $asi;
	}

	function add_ref_key() { // Select reference key and type
		echo "<form action=\"" . $_SERVER["PHP_SELF"] . "\" method=\"post\" enctype=\"multipart/form-data\"><fieldset>\n";
		echo "<input type=\"hidden\" name=\"action\" value=\"add_ref_check_key\" />\n";
		echo "<h3>Reference key</h3>\n";
		echo "<input type=\"text\" name=\"bibkey\" size=\"15\" maxlength=\"15\" />\n";
		echo "<h3>Reference type</h3><select name=\"reference\">\n";
	    $c = $this->config; require("db-connect.inc.php");
	    $result = $cnx->query("select type from btw_reference order by type");
	    foreach ($result as $row) echo "<option>" . $row["type"] . "</option>\n";	
		echo "</select>\n";
		echo "<p><input type=\"reset\" name=\"Reset\" value=\"Reset\" />\n";
		echo "<input type=\"submit\" name=\"Submit\" value=\"Submit\" /></p>\n";
		echo "</fieldset></form>\n\n";
	}
	
	function select_authed($action, $table, $onvalid, $bibkey, $reference, $subaction = "", $name = "") { // Add/remove authors/editors to a reference
		$c = $this->config; require("db-connect.inc.php");
		//echo "<p>Working on table: $table</p>\n"; // Debug
		// Actions
		if (($subaction == "Add_$table") || ($subaction == "Remove_$table")) {
			// Fetch ID
			$t = new Table($this->config, "author");
			$id = $t->get_id($name); 
			if ($subaction == "Add_$table") {			    
				$result = $cnx->query("select count(*) as c from btw_$table where bibkey='$bibkey'");
				$row = $result->fetch(PDO::FETCH_ASSOC);
				if ($row["c"]) {
					$result2 = $cnx->query("select max(rank)+1 as r from btw_$table where bibkey='$bibkey'");
					$row2 = $result2->fetch(PDO::FETCH_ASSOC);
					$rank = $row2["r"];
				} else $rank = 1;
				$cnx->exec("insert into btw_$table values ('$bibkey', $id, $rank)");
			} else $cnx->exec("delete from btw_$table where bibkey='$bibkey' and id=$id"); // Remove
		}
		// Already selected authors
		$q = "select name from btw_author as a, btw_$table as b where a.id=b.id and bibkey='$bibkey' order by rank";
		$rowcount = $cnx->query($q);
		$nauth = count($rowcount->fetchAll());
		if ($nauth) {
			$result = $cnx->query($q);
			echo "<h3>Selected ";
			if (($table == "proc_editors") || ($table == "book_editors")) echo "editors";
			else echo $table;
			echo "</h3><ul>\n\n";
			foreach ($result as $row) echo "<li>" . $row["name"] . "</li>\n";
			echo "</ul>\n\n";
		} else {
			echo "<h3>Select ";
			if (($table == "proc_editors") || ($table == "book_editors")) echo "editors";
			else echo $table;
			if (($reference == "book") || (($reference == "inproceedings") && ($table == "editors"))) echo "<span class=\"small\"> (optional)</span>";
			echo "</h3>\n\n";
		}
		// Add author
		echo "<form action=\"" . $_SERVER["PHP_SELF"] . "\" method=\"post\" enctype=\"multipart/form-data\"><fieldset>\n";
		echo "<input type=\"hidden\" name=\"action\" value=\"$action\" />\n";
		echo "<input type=\"hidden\" name=\"onvalid\" value=\"$onvalid\" />\n";
		echo "<input type=\"hidden\" name=\"bibkey\" value=\"$bibkey\" />\n";
		echo "<input type=\"hidden\" name=\"reference\" value=\"$reference\" />\n";
		echo "<input type=\"hidden\" name=\"table\" value=\"$table\" />\n";
		echo "<select name=\"name\">\n";
		$result = $cnx->query("select name from btw_author order by name");
		foreach ($result as $row) echo "<option>" . $row["name"] . "</option>\n";
		echo "</select>\n";
		echo "<input type=\"hidden\" name=\"subaction\" value=\"Add_$table\" />\n";
		echo "<input type=\"submit\" name=\"add\" value=\"Add\" />\n";
		echo "</fieldset></form>\n\n";
		// Remove author
		echo "<form action=\"" . $_SERVER["PHP_SELF"] . "\" method=\"post\" enctype=\"multipart/form-data\" /><fieldset>\n";
		echo "<input type=\"hidden\" name=\"action\" value=\"$action\" />\n";
		echo "<input type=\"hidden\" name=\"onvalid\" value=\"$onvalid\" />\n";
		echo "<input type=\"hidden\" name=\"bibkey\" value=\"$bibkey\" />\n";
		echo "<input type=\"hidden\" name=\"reference\" value=\"$reference\" />\n";
		echo "<input type=\"hidden\" name=\"table\" value=\"$table\" />\n";		
		echo "<select name=\"name\">\n";
		$result = $cnx->query("select name from btw_author as a, btw_$table as b where a.id=b.id and bibkey='$bibkey' order by rank");
		foreach ($result as $row) echo "<option>" . $row["name"] . "</option>\n";
		echo "</select>\n";
		echo "<input type=\"hidden\" name=\"subaction\" value=\"Remove_$table\" />\n";
		echo "<input type=\"submit\" name=\"remove\" value=\"Remove\" />\n";
		echo "</fieldset></form>\n\n";		
		// End
		//echo "<p>onvalid = $onvalid   table = $table</p>\n"; // Debug
		if (($onvalid) && (
							(
								(
									($table == "authors") || ($table == "proc_editors")
								)
								&&
								($nauth)
							) 
							|| (($table == "authors") && ($reference == "book"))
							|| ($table == "editors") 
							|| ($table == "book_editors")
						)
			) {
			switch ($reference) {
				case "book":				$nexttable = "book_editors"; break;
				case "proceedings":		$nexttable = "proc_editors"; break;
				case "inproceedings":	$nexttable = "editors"; break;
				default:						$nexttable = "";
			}
			echo "<form action=\"" . $_SERVER["PHP_SELF"] . "\" method=\"post\" enctype=\"multipart/form-data\" /><fieldset>\n";
			echo "<input type=\"hidden\" name=\"action\" value=\"$onvalid\" />\n";
			echo "<input type=\"hidden\" name=\"bibkey\" value=\"$bibkey\" />\n";
			echo "<input type=\"hidden\" name=\"reference\" value=\"$reference\" />\n";
			echo "<input type=\"hidden\" name=\"table\" value=\"$nexttable\" />\n";					
			echo "<input type=\"submit\" name=\"submit\" value=\"Proceed\" />\n";
			echo "</fieldset></form>\n\n";
		}
	}		
	
	function add_ref_check_key($bibkey, $reference) { // Check reference key		
		$c = $this->config; require("db-connect.inc.php");
		$result = $cnx->query("select count(*) as c from btw_paper where bibkey='$bibkey'");
		$row = $result->fetch(PDO::FETCH_ASSOC);
		if (!$row["c"]) { // Resume -> select authors
			if (($reference != "inproceedings") && ($reference != "book")) $onvalid = "add_ref_form";
			else $onvalid = "add_ref_editors";
			if ($reference != "proceedings") $this->select_authed("add_ref_authors", "authors", $onvalid, $bibkey, $reference);
			else $this->select_authed("add_ref_editors", "proc_editors", $onvalid, $bibkey, $reference);
		} else { // Error
			echo "<p>This key already exists in the database, please select another one.</p>\n\n";
			$this->return_button();
		}	
	}
	
	function next_step($id, $step, $req = "", $reference = "", $scope = "", $bibkey = "", $author = "", $editor = "", $title = "", $year = "", $month = "", $note = "", $journal = "", $volume = "", $number = "", $pages = "", $series = "", $publisher = "", $edition = "", $booktitle = "", $organization = "", $school = "", $howpublished = "", $institution = "", $type = "", $address = "", $auth_id_list = "", $index_a = 0, $ed_id_list = "", $index_e = 0, $idseries = 0, $idinst = 0, $abstract = "", $keywords = "", $subaction = "", $variant_param = "") { // Next step form during BibTeX import process
		if ($subaction) $msg = $subaction;
		else {
			if ($step == "load") $msg = "Proceed to next entry";
			else $msg = "Proceed to next step";
		}
		echo "<form action=\"" . $_SERVER["PHP_SELF"] . "\" method=\"post\" enctype=\"multipart/form-data\"><fieldset>\n";
		echo "<input type=\"hidden\" name=\"id\" value=\"$id\" />\n";
		echo "<input type=\"hidden\" name=\"action\" value=\"import_$step\" />\n";
		echo "<input type=\"hidden\" name=\"req\" value=\"$req\" />\n";
		echo "<input type=\"hidden\" name=\"reference\" value=\"$reference\" />\n";
		echo "<input type=\"hidden\" name=\"scope\" value=\"$scope\" />\n";
		echo "<input type=\"hidden\" name=\"bibkey\" value=\"$bibkey\" />\n";
		echo "<input type=\"hidden\" name=\"author\" value=\"$author\" />\n";
		echo "<input type=\"hidden\" name=\"editor\" value=\"$editor\" />\n";
		echo "<input type=\"hidden\" name=\"title\" value=\"$title\" />\n";
		echo "<input type=\"hidden\" name=\"year\" value=\"$year\" />\n";
		echo "<input type=\"hidden\" name=\"month\" value=\"$month\" />\n";
		echo "<input type=\"hidden\" name=\"note\" value=\"$note\" />\n";
		echo "<input type=\"hidden\" name=\"journal\" value=\"$journal\" />\n";
		echo "<input type=\"hidden\" name=\"volume\" value=\"$volume\" />\n";
		echo "<input type=\"hidden\" name=\"number\" value=\"$number\" />\n";
		echo "<input type=\"hidden\" name=\"pages\" value=\"$pages\" />\n";
		echo "<input type=\"hidden\" name=\"series\" value=\"$series\" />\n";
		echo "<input type=\"hidden\" name=\"publisher\" value=\"$publisher\" />\n";
		echo "<input type=\"hidden\" name=\"edition\" value=\"$edition\" />\n";
		echo "<input type=\"hidden\" name=\"booktitle\" value=\"$booktitle\" />\n";
		echo "<input type=\"hidden\" name=\"organization\" value=\"$organization\" />\n";
		echo "<input type=\"hidden\" name=\"school\" value=\"$school\" />\n";
		echo "<input type=\"hidden\" name=\"howpublished\" value=\"$howpublished\" />\n";
		echo "<input type=\"hidden\" name=\"institution\" value=\"$institution\" />\n";
		echo "<input type=\"hidden\" name=\"type\" value=\"$type\" />\n";
		echo "<input type=\"hidden\" name=\"address\" value=\"$address\" />\n";
		echo "<input type=\"hidden\" name=\"auth_id_list\" value=\"$auth_id_list\" />\n";
		echo "<input type=\"hidden\" name=\"index_a\" value=\"$index_a\" />\n";
		echo "<input type=\"hidden\" name=\"ed_id_list\" value=\"$ed_id_list\" />\n";
		echo "<input type=\"hidden\" name=\"index_e\" value=\"$index_e\" />\n";
		echo "<input type=\"hidden\" name=\"idseries\" value=\"$idseries\" />\n";
		echo "<input type=\"hidden\" name=\"idinst\" value=\"$idinst\" />\n";
		echo "<input type=\"hidden\" name=\"abstract\" value=\"$abstract\" />\n";
		echo "<input type=\"hidden\" name=\"keywords\" value=\"$keywords\" />\n";				
		echo "<input type=\"hidden\" name=\"variant_param\" value=\"$variant_param\" />\n";
		echo "<input type=\"hidden\" name=\"subaction\" value=\"$subaction\" />\n";
		echo "<input type=\"submit\" value=\"$msg\" />\n";
		echo "</fieldset></form>\n\n";
	}	
	
	function bibtex_import_phase1($id = "", $tmpfile = "", $filename = "") { 
		$c = $this->config; require("db-connect.inc.php");
		// Step 1: Load file into database temporary table
		//echo "<p>id=$id</p>\n"; // Debug
		if (!$id) { 
			echo "<p>Importing file <strong>$filename</strong>.</p>\n\n";
			$entry = "";
			$i = 0;
			$cnx->exec("delete from btw_import");
			if ($tmpfile) {
				$fd = fopen($tmpfile, "r");
				// Move on first entry marker (@)
				if (!feof($fd)) $l = trim(fgets($fd)); 
				while (($l[0] != "@") && (!feof($fd))) $l = trim(fgets($fd)); 
				while (!feof($fd)) {
					$entry = $l;
					$i++;
					// Move on next entry marker 
					$l = trim(fgets($fd));
					while (($l[0]!="@") && (!feof($fd))) {	
						if ($l[0] != "%") $entry .= $l;
						$l = trim(fgets($fd));
					}
					// Store entry into DB
					$q = "insert into btw_import values ($i, '" . addslashes($entry) ."')";
					$cnx->exec($q);
					$entry = "";
				}
				fclose($fd);
			}
		} else { // Delete previous entry if no error was detected
			$cnx->exec("delete from btw_import where id=$id");
		}
		// Steps 2-N: Parse entries
		$q = "select * from btw_import";
		$rowcount = $cnx->query($q);
		$n = count($rowcount->fetchAll());
		if ($n <=1 ) $entlab = "entry"; else $entlab = "entries";
		echo "<p>$n BibTeX $entlab remaining.</p>\n\n";
		if ($n) { // Entries remain to process
			// Fetch first entry
			$result = $cnx->query($q);
			$row = $result->fetch(PDO::FETCH_ASSOC);
			$id = $row["id"];
			$entry = $row["entry"];
			echo "<p><strong>Current entry:</strong> $entry</p>\n\n";
			// Parse entry
			$entry = str_replace('"', '}', $entry);
			$item = explode("},", $entry);
			// Reference type, key, and author
			$line2 = explode(",", $item[0]);
			$item[0] = $line2[1]; // Store author in $item[0]
			$line = explode("{", $line2[0]);
			$reference = strtolower(trim(str_replace("@", " ", $line[0])));
			$bibkey = trim($line[1]);
			// All other items
			$errno = 0;
			$errm = "";
			for ($i = 0; !empty($item[$i]); $i++) {
				$line = explode("=", $item[$i]);  
				$left = strtolower(trim($line[0]));
				$right = trim(str_replace("}", " ", str_replace("{", " ", $line[1])));
				switch($left) {
					case "author": 		$author = $right; break;
					case "editor": 		$editor = $right; break;
					case "title":			$title = $right; break;
					case "year": 			$year = $right; break;
					case "month": 			$month = $right; break;
					case "note":	 		$note = $right; break;
					case "journal": 		$journal = $right; break;
					case "volume":			$volume = $right; break;
					case "number": 		$number = $right; break;
					case "pages":	 		$pages = $right; break;
					case "series":			$series = $right; break;
					case "publisher":		$publisher = $right; break;
					case "edition":	 	$edition = $right; break;
					case "booktitle":		$booktitle = $right; break;
					case "organization":	$organization = $right; break;
					case "school":	 		$school = $right; break;
					case "howpublished":	$howpublished = $right; break;
					case "institution":	$institution = $right; break;
					case "type":	 		$type = $right; break;
					case "address":		$address = $right; break;
					case "abstract":		$abstract = $right; break;
					case "keywords":		$keywords = $right; break;
					default: {
						$errno++;
						$errm .= $left . " ";
					}
				}
			}
			/* echo "<p>reference = $reference<br/>
			   bibkey = $bibkey<br/>
				author = $author<br/>
				editor = $editor<br/>
				title = $title<br/>
				year = $year<br/>
				month = $month<br/>
 				note = $note<br/>
				journal = $journal<br/>
				volume = $volume<br/>
				number = $number<br/>
				pages = $pages<br/>
				series = $series<br/>
				publisher = $publisher<br/>
				edition = $edition<br/>
				booktitle = $booktitle<br/>
				organization = $organization<br/>
				school = $school<br/>
				howpublished = $howpublished<br/>
				institution = $institution<br/>
				type = $type<br/>
				address = $address<br/>
				$errno errors / Fields: $errm</p>";	// Debug	*/
			// Test if reference already exists in DB
			$result = $cnx->query("select count(*) as c from btw_paper where bibkey='$bibkey'");
			$row = $result->fetch(PDO::FETCH_ASSOC);
			$exists = $row["c"];
			if ($exists) echo "<p><strong>Error:</strong> A reference labeled \"$bibkey\" already exists in the database. Current entry will be discarded.</p>\n\n";
			// Test if reference type is supported
			$ref_ok = 0;
			$result = $cnx->query("select type from btw_reference");
			foreach ($result as $row) {
				if ($reference == $row["type"]) {
					$ref_ok = 1;
					break;
				}
			}      
			if (!$ref_ok) echo "<p><strong>Error:</strong> Reference type \"$reference\" is not supported. Current entry will be discarded.</p>\n\n";
			//
			if ($errno) echo "<p><strong>Error:</strong> Field type(s) \"$errm\" is(are) not supported. Current entry will be discarded.</p>";
			if ((!$exists) && ($ref_ok) && (!$errno)) {
				// Select scope
				echo "<h3>Imported fields control</h3>\n\n";
				echo "<form action=\"" . $_SERVER["PHP_SELF"] . "\" method=\"post\" enctype=\"multipart/form-data\"><fieldset>\n";
				echo "<input type=\"hidden\" name=\"id\" value=\"$id\" />\n";
				echo "<input type=\"hidden\" name=\"action\" value=\"import_requisites\" />\n";
				echo "<input type=\"hidden\" name=\"req\" value=\"INIT\" />\n";
				echo "<table>\n";
				echo "<tr><td>Reference type : </td><td><select name=\"reference\">\n";
				$result = $cnx->query("select type from btw_reference order by type");
				foreach ($result as $row) {
					echo "<option";
					if ($row["type"] == $reference) echo " selected=\"selected\"";
					echo ">" . $row["type"] . "</option>\n";
				}
				echo "</select></td></tr>\n";
				echo "<tr><td>Reference key : </td><td><input type=\"text\" name=\"bibkey\" size=\"100\" maxlength=\"255\" value=\"$bibkey\" /></td></tr>\n";
				echo "<tr><td>Author : </td><td><input type=\"text\" name=\"author\" size=\"100\" maxlength=\"255\" value=\"$author\" /></td></tr>\n";
				echo "<tr><td>Editor : </td><td><input type=\"text\" name=\"editor\" size=\"100\" maxlength=\"255\" value=\"$editor\" /></td></tr>\n";
				echo "<tr><td>Title : </td><td><input type=\"text\" name=\"title\" size=\"100\" maxlength=\"255\" value=\"$title\" /></td></tr>\n";
				echo "<tr><td>Year : </td><td><input type=\"text\" name=\"year\" size=\"100\" maxlength=\"255\" value=\"$year\" /></td></tr>\n";
				echo "<tr><td>Month : </td><td><input type=\"text\" name=\"month\" size=\"100\" maxlength=\"255\" value=\"$month\" /></td></tr>\n";
				echo "<tr><td>Note : </td><td><input type=\"text\" name=\"note\" size=\"100\" maxlength=\"255\" value=\"$note\" /></td></tr>\n";
				echo "<tr><td>Journal : </td><td><input type=\"text\" name=\"journal\" size=\"100\" maxlength=\"255\" value=\"$journal\" /></td></tr>\n";
				echo "<tr><td>Volume : </td><td><input type=\"text\" name=\"volume\" size=\"100\" maxlength=\"255\" value=\"$volume\" /></td></tr>\n";
				echo "<tr><td>Number : </td><td><input type=\"text\" name=\"number\" size=\"100\" maxlength=\"255\" value=\"$number\" /></td></tr>\n";
				echo "<tr><td>Pages : </td><td><input type=\"text\" name=\"pages\" size=\"100\" maxlength=\"255\" value=\"$pages\" /></td></tr>\n";
				echo "<tr><td>Series : </td><td><input type=\"text\" name=\"series\" size=\"100\" maxlength=\"255\" value=\"$series\" /></td></tr>\n";
				echo "<tr><td>Publisher : </td><td><input type=\"text\" name=\"publisher\" size=\"100\" maxlength=\"255\" value=\"$publisher\" /></td></tr>\n";
				echo "<tr><td>Edition : </td><td><input type=\"text\" name=\"edition\" size=\"100\" maxlength=\"255\" value=\"$edition\" /></td></tr>\n";
				echo "<tr><td>Booktitle : </td><td><input type=\"text\" name=\"booktitle\" size=\"100\" maxlength=\"255\" value=\"$booktitle\" /></td></tr>\n";
				echo "<tr><td>Organization : </td><td><input type=\"text\" name=\"organization\" size=\"100\" maxlength=\"255\" value=\"$organization\" /></td></tr>\n";
				echo "<tr><td>School : </td><td><input type=\"text\" name=\"school\" size=\"100\" maxlength=\"255\" value=\"$school\" /></td></tr>\n";
				echo "<tr><td>Howpublished : </td><td><input type=\"text\" name=\"howpublished\" size=\"100\" maxlength=\"255\" value=\"$howpublished\" /></td></tr>\n";
				echo "<tr><td>Institution : </td><td><input type=\"text\" name=\"institution\" size=\"100\" maxlength=\"255\" value=\"$institution\" /></td></tr>\n";
				echo "<tr><td>Type : </td><td><input type=\"text\" name=\"type\" size=\"100\" maxlength=\"255\" value=\"$type\" /></td></tr>\n";
				echo "<tr><td>Address : </td><td><input type=\"text\" name=\"address\" size=\"100\" maxlength=\"255\" value=\"$address\" /></td></tr>\n";
				echo "<tr><td>Reference scope : </td><td><select name=\"scope\"><option>International</option><option>National</option></select></td></tr>\n";
				echo "<tr><td>Abstract :</td><td><textarea name=\"abstract\" rows=\"5\" cols=\"62\">$abstract</textarea></td>\n";
				echo "<tr><td>Keywords : </td><td><input type=\"text\" name=\"keywords\" size=\"100\" maxlength=\"255\" value=\"$keywords\" /></td></tr>\n";				
				echo "</table>\n";
				echo "<input type=\"hidden\" name=\"index_a\" value=\"0\" />\n";
				echo "<input type=\"hidden\" name=\"index_e\" value=\"0\" />\n";
				echo "<p><input type=\"submit\" value=\"Confirm\" /></p>\n";
				echo "</fieldset></form>\n\n";
			} else $this->next_step($id, "load");
		} else $this->return_button();	
	}
	
	function import_requisites($author, $editor, $journal, $series, $publisher, $school, $institution, $organization) {
		// List requisites that must be in the DB before the reference is stored
		$req = "";
		$line = explode(" and ", $author);
		for ($i = 0; !empty($line[$i]); $i++) $req .= "auth";
		$line = explode(" and ", $editor);
		for ($i = 0; !empty($line[$i]); $i++) $req .= "edit";
		if ($journal) $req .= "jour";
		if ($series) $req .= "seri";
		if ($publisher) $req .= "publ";
		if ($school) $req .= "scho";
		if ($institution) $req .= "inst";
		if ($organization) $req .= "orga";
		//echo "<p>Requisites: $req</p>"; // Debug	
		return $req;
	}		
	
	function bibtex_import_phase2($id, $req, $reference, $scope, $bibkey, $author, $editor, $title, $year, $month, $note, $journal, $volume, $number, $pages, $series, $publisher, $edition, $booktitle, $organization, $school, $howpublished, $institution, $type, $address, $auth_id_list, $index_a, $ed_id_list, $index_e, $idseries, $idinst, $abstract = "", $keywords = "", $subaction = "", $variant_param = "") {
		$c = $this->config; require("db-connect.inc.php");
		echo "<p>Importing reference entitled <strong>$title</strong>.</p>\n\n";
		switch ($subaction) { // Database operation required	
			case "Select author":
				$auth_id_list .= $variant_param . " "; // Variant is author ID
			break;
			case "Select editor":
				$ed_id_list .= $variant_param . " "; // Variant is editor ID
			break;
			case "Insert author":
				$t = new Table($this->config, "author");
				$auth_id = $t->get_new_id();
				$cnx->exec("insert into btw_author values($auth_id, '" . addslashes($variant_param) . "')");	// Variant is author name
				$auth_id_list .= $auth_id . " ";
			break;
			case "Insert editor":
				$t = new Table($this->config, "author");
				$auth_id = $t->get_new_id();
				$cnx->exec("insert into btw_author values($auth_id, '" . addslashes($variant_param) . "')");	// Variant is editor name
				$ed_id_list .= $auth_id . " ";
			break;
			case "Insert journal":
				$t = new Table($this->config, "series");
				$idseries = $t->get_new_id();
				$cnx->exec("insert into btw_series values($idseries, '" . addslashes($variant_param) . "', NULL)");	// Variant is journal name
			break;
			case "Insert series":
				$t = new Table($this->config, "series");
				$idseries = $t->get_new_id();
				$cnx->exec("insert into btw_series values($idseries, '" . addslashes($variant_param) . "', NULL)");	// Variant is series name
			break;
			case "Insert publisher":
				$t = new Table($this->config, "institution");
				$idinst = $t->get_new_id();
				$cnx->exec("insert into btw_institution values($idinst, '" . addslashes($variant_param) . "', '" . addslashes($address) . "', NULL)");	// Variant is publisher name
			break;
			case "Insert institution":
				$t = new Table($this->config, "institution");
				$idinst = $t->get_new_id();
				$cnx->exec("insert into btw_institution values($idinst, '" . addslashes($variant_param) . "', '" . addslashes($address) . "', NULL)");	// Variant is institution name
			break;
			case "Insert school":
				$t = new Table($this->config, "institution");
				$idinst = $t->get_new_id();
				$cnx->exec("insert into btw_institution values($idinst, '" . addslashes($variant_param) . "', '" . addslashes($address) . "', NULL)");	// Variant is school name
			break;
			case "Insert organization":
				$t = new Table($this->config, "institution");
				$idinst = $t->get_new_id();
				$cnx->exec("insert into btw_institution values($idinst, '" . addslashes($variant_param) . "', '" . addslashes($address) . "', NULL)");	// Variant is organization name
			break;
		}
		if ($req) { // Requisites are remaining
			$creq = substr($req, 0, 4); // Current
			$req = substr($req, 4);
			//echo "<p>creq = $creq / req = $req</p>"; // Debug
			switch ($creq) {
				case "auth": // Author
				$this->search_author("author", $id, "requisites", $req, $reference, $scope, $bibkey, $author, $editor, $title, $year, $month, $note, $journal, $volume, $number, $pages, $series, $publisher, $edition, $booktitle, $organization, $school, $howpublished, $institution, $type, $address, $auth_id_list, $index_a, $ed_id_list, $index_e, $idseries, $idinst, $abstract, $keywords);
			break;
			case "edit": // Editor
				$this->search_author("editor", $id, "requisites", $req, $reference, $scope, $bibkey, $author, $editor, $title, $year, $month, $note, $journal, $volume, $number, $pages, $series, $publisher, $edition, $booktitle, $organization, $school, $howpublished, $institution, $type, $address, $auth_id_list, $index_a, $ed_id_list, $index_e, $idseries, $idinst, $abstract, $keywords);
			break;
			case "jour": // Journal
				$this->search_series("journal", $id, "requisites", $req, $reference, $scope, $bibkey, $author, $editor, $title, $year, $month, $note, $journal, $volume, $number, $pages, $series, $publisher, $edition, $booktitle, $organization, $school, $howpublished, $institution, $type, $address, $auth_id_list, $index_a, $ed_id_list, $index_e, $idseries, $idinst, $abstract, $keywords);
			break;
			case "seri": // Series
				$this->search_series("series", $id, "requisites", $req, $reference, $scope, $bibkey, $author, $editor, $title, $year, $month, $note, $journal, $volume, $number, $pages, $series, $publisher, $edition, $booktitle, $organization, $school, $howpublished, $institution, $type, $address, $auth_id_list, $index_a, $ed_id_list, $index_e, $idseries, $idinst, $abstract, $keywords);
			break;
			case "publ": // Publisher
				$this->search_inst("publisher", $id, "requisites", $req, $reference, $scope, $bibkey, $author, $editor, $title, $year, $month, $note, $journal, $volume, $number, $pages, $series, $publisher, $edition, $booktitle, $organization, $school, $howpublished, $institution, $type, $address, $auth_id_list, $index_a, $ed_id_list, $index_e, $idseries, $idinst, $abstract, $keywords);
			break;
			case "scho": // School
				$this->search_inst("school", $id, "requisites", $req, $reference, $scope, $bibkey, $author, $editor, $title, $year, $month, $note, $journal, $volume, $number, $pages, $series, $publisher, $edition, $booktitle, $organization, $school, $howpublished, $institution, $type, $address, $auth_id_list, $index_a, $ed_id_list, $index_e, $idseries, $idinst, $abstract, $keywords);
			break;
			case "inst": // Institution
				$this->search_inst("institution", $id, "requisites", $req, $reference, $scope, $bibkey, $author, $editor, $title, $year, $month, $note, $journal, $volume, $number, $pages, $series, $publisher, $edition, $booktitle, $organization, $school, $howpublished, $institution, $type, $address, $auth_id_list, $index_a, $ed_id_list, $index_e, $idseries, $idinst, $abstract, $keywords);
			break;
			case "orga": // Organization
				$this->search_inst("organization", $id, "requisites", $req, $reference, $scope, $bibkey, $author, $editor, $title, $year, $month, $note, $journal, $volume, $number, $pages, $series, $publisher, $edition, $booktitle, $organization, $school, $howpublished, $institution, $type, $address, $auth_id_list, $index_a, $ed_id_list, $index_e, $idseries, $idinst, $abstract, $keywords);
			break;
		} 
    } else $this->next_step($id, "insert", $req, $reference, $scope, $bibkey, $author, $editor, $title, $year, $month, $note, $journal, $volume, $number, $pages, $series, $publisher, $edition, $booktitle, $organization, $school, $howpublished, $institution, $type, $address, $auth_id_list, $index_a, $ed_id_list, $index_e, $idseries, $idinst, $abstract, $keywords);	
	}
}
?>